@@ -91,10 +91,14 @@ void create_about_window()
     gtk_box_pack_start(GTK_BOX(vbox), separator, FALSE, FALSE, 3);
 
 #if GTK_CHECK_VERSION(2,18,9)
-   GtkWidget *label = gtk_label_new(_(_L("<a href='http://hyperrate.com?eid=67'>點選連結前往 gcin 討論區</a>\n"
-_L("<a href='http://hyperrate.com?eid=215'>gcin也有 Windows版</a>\n")
-_L("<a href='"LOG_URL"'>gcin改變記錄</a>\n")
-)));
+   char tmp[512];
+   sprintf(tmp, "<a href='http://hyperrate.com?eid=67'>%s</a>\n"
+                "<a href='http://hyperrate.com?eid=215'>%s</a>\n"
+                "<a href='"LOG_URL"'>%s</a>\n",
+                _(_L("點選連結前往 gcin 討論區")),
+                _(_L("gcin也有 Windows版")),
+                _(_L("gcin改變記錄")));
+   GtkWidget *label = gtk_label_new(tmp);
    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
     gtk_box_pack_start(GTK_BOX(vbox), label, FALSE, FALSE, 0);
 #else
