@@ -24,10 +24,7 @@
 
 #include <gdk/gdkx.h>
 #include <X11/Xatom.h>
-#include <gtk/gtk.h>
-
-#define _(x) x
-#define N_(x) x
+#include "gcin.h"
 
 #define SYSTEM_TRAY_REQUEST_DOCK    0
 #define SYSTEM_TRAY_BEGIN_MESSAGE   1
@@ -283,7 +280,7 @@ egg_tray_icon_send_manager_message (EggTrayIcon *icon,
   ev.window = window;
   ev.message_type = icon->system_tray_opcode_atom;
   ev.format = 32;
-  ev.data.l[0] = gdk_x11_get_server_time (GTK_WIDGET (icon)->window);
+  ev.data.l[0] = gdk_x11_get_server_time (gtk_widget_get_window(GTK_WIDGET (icon)));
   ev.data.l[1] = message;
   ev.data.l[2] = data1;
   ev.data.l[3] = data2;
@@ -352,7 +349,7 @@ egg_tray_icon_update_manager_window (EggTrayIcon *icon,
 
       gdk_window_add_filter (gdkwin, egg_tray_icon_manager_filter, icon);
 
-      if (dock_if_realized && GTK_WIDGET_REALIZED (icon))
+      if (dock_if_realized && GTK_WIDGET_REALIZED (GTK_WIDGET (icon)))
 	egg_tray_icon_send_dock_request (icon);
 
       egg_tray_icon_get_orientation_property (icon);
@@ -380,32 +377,64 @@ egg_tray_icon_manager_window_destroyed (EggTrayIcon *icon)
   egg_tray_icon_update_manager_window (icon, TRUE);
 }
 
+#if !GTK_CHECK_VERSION(2,91,0)
 static gboolean
 transparent_expose_event (GtkWidget *widget, GdkEventExpose *event, gpointer user_data)
 {
-	gdk_window_clear_area (widget->window, event->area.x, event->area.y,
+	gdk_window_clear_area (gtk_widget_get_window(widget), event->area.x, event->area.y,
 	                      event->area.width, event->area.height);
 	return FALSE;
 }
+#else
+static gboolean
+transparent_expose_event (GtkWidget *widget, cairo_t *event, gpointer user_data)
+{
+	guint width, height;
+	GdkRGBA color;
+	width = gtk_widget_get_allocated_width (widget);
+	height = gtk_widget_get_allocated_height (widget);
+	cairo_arc (event, width / 2.0, height / 2.0,
+	           MIN (width, height) / 2.0, 0, 2 * G_PI);
+	gtk_style_context_get_color (gtk_widget_get_style_context (widget),
+	                             0, &color);
+	gdk_cairo_set_source_rgba (event, &color);
+	cairo_fill (event);
+	return FALSE;
+}
+#endif
 
 static void
 make_transparent_again (GtkWidget *widget, GtkStyle *previous_style,
                        gpointer user_data)
 {
-	gdk_window_set_back_pixmap(widget->window, NULL, TRUE);
+#if !GTK_CHECK_VERSION(2,91,0)
+	gdk_window_set_back_pixmap(gtk_widget_get_window(widget), NULL, TRUE);
+#else
+	gdk_window_set_background_pattern(gtk_widget_get_window(widget), NULL);
+#endif
 }
 
 static void
 make_transparent (GtkWidget *widget, gpointer user_data)
 {
+#if GTK_CHECK_VERSION(2,19,3)
+	if (GTK_WIDGET_NO_WINDOW (widget) || gtk_widget_get_app_paintable (widget))
+#else
 	if (GTK_WIDGET_NO_WINDOW (widget) || GTK_WIDGET_APP_PAINTABLE (widget))
+#endif
 		return;
 
 	gtk_widget_set_app_paintable (widget, TRUE);
 	gtk_widget_set_double_buffered (widget, FALSE);
-	gdk_window_set_back_pixmap (widget->window, NULL, TRUE);
+#if !GTK_CHECK_VERSION(2,91,0)
+	gdk_window_set_back_pixmap (gtk_widget_get_window(widget), NULL, TRUE);
 	g_signal_connect (widget, "expose_event",
 	                 G_CALLBACK (transparent_expose_event), NULL);
+#else
+	gdk_window_set_background_pattern (gtk_widget_get_window(widget), NULL);
+	g_signal_connect (widget, "draw",
+	                 G_CALLBACK (transparent_expose_event), NULL);
+#endif
 	g_signal_connect_after (widget, "style_set",
 	                       G_CALLBACK (make_transparent_again), NULL);
 }
