@@ -5,7 +5,10 @@
 #include <X11/extensions/XTest.h>
 #endif
 #include "gst.h"
+#include "pho.h"
 #include "im-client/gcin-im-client-attr.h"
+#include "win1.h"
+#include "gcin-module.h"
 
 #define STRBUFLEN 64
 
@@ -23,10 +26,14 @@ char *output_buffer;
 int output_bufferN;
 static char *output_buffer_raw, *output_buffer_raw_bak;
 static int output_buffer_rawN;
-gboolean test_mode;
 #if WIN32
+gboolean test_mode;
 int last_input_method;
 #endif
+void set_wselkey(char *s);
+void gtab_set_win1_cb();
+
+gboolean old_capslock_on;
 
 
 void init_gtab(int inmdno);
@@ -313,7 +320,17 @@ void reset_current_in_win_xy()
   current_in_win_x = current_in_win_y = -1;
 }
 
-void hide_win_anthy();
+GCIN_module_callback_functions *module_cb1(ClientState *cs)
+{
+  return inmd[cs->in_method].mod_cb_funcs;
+}
+
+GCIN_module_callback_functions *module_cb()
+{
+  if (!current_CS)
+    return NULL;
+  return module_cb1(current_CS);
+}
 
 void hide_in_win(ClientState *cs)
 {
@@ -345,11 +362,9 @@ void hide_in_win(ClientState *cs)
     case method_type_INT_CODE:
       hide_win_int();
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
-      hide_win_anthy();
+    case method_type_MODULE:
+      module_cb1(cs)->module_hide_win();
       break;
-#endif
     default:
       hide_win_gtab();
   }
@@ -360,7 +375,7 @@ void hide_in_win(ClientState *cs)
 void show_win_pho();
 void show_win0();
 void show_win_int();
-void show_win_gtab(), show_win_anthy();
+void show_win_gtab();
 
 void check_CS()
 {
@@ -396,11 +411,9 @@ void show_in_win(ClientState *cs)
     case method_type_INT_CODE:
       show_win_int();
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
-      show_win_anthy();
+    case method_type_MODULE:
+      module_cb1(cs)->module_show_win();
       break;
-#endif
     default:
       show_win_gtab();
   }
@@ -414,7 +427,6 @@ void move_win_gtab(int x, int y);
 void move_win_int(int x, int y);
 void move_win0(int x, int y);
 void move_win_pho(int x, int y);
-void move_win_anthy(int x, int y);
 
 void move_in_win(ClientState *cs, int x, int y)
 {
@@ -450,11 +462,9 @@ void move_in_win(ClientState *cs, int x, int y)
     case method_type_INT_CODE:
       move_win_int(x, y);
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
-      move_win_anthy(x, y);
+    case method_type_MODULE:
+      module_cb1(cs)->module_move_win(x, y);
       break;
-#endif
     default:
       if (!cs->in_method)
         return;
@@ -647,7 +657,6 @@ void disp_im_half_full()
 
 void flush_tsin_buffer();
 void reset_gtab_all();
-void set_tsin_pho_mode();
 void set_tsin_pho_mode0(ClientState *cs);
 
 //static u_int orig_caps_state;
@@ -732,7 +741,6 @@ void toggle_im_enabled()
 void get_win_gtab_geom();
 void get_win0_geom();
 void get_win_pho_geom();
-void get_win_anthy_geom();
 void get_win_int_geom();
 
 void update_active_in_win_geom()
@@ -748,13 +756,11 @@ void update_active_in_win_geom()
       break;
 #endif
     case method_type_INT_CODE:
-	  get_win_int_geom();
+      get_win_int_geom();
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
-      get_win_anthy_geom();
+    case method_type_MODULE:
+      module_cb()->module_get_win_geom();
       break;
-#endif
     default:
       get_win_gtab_geom();
       break;
@@ -762,7 +768,6 @@ void update_active_in_win_geom()
 }
 
 extern GtkWidget *gwin_pho, *gwin0, *gwin_gtab, *gwin_int;
-int anthy_visible();
 
 gboolean win_is_visible()
 {
@@ -777,10 +782,8 @@ gboolean win_is_visible()
 #endif
     case method_type_INT_CODE:
 	  return gwin_int && GTK_WIDGET_VISIBLE(gwin_int);
-#if USE_ANTHY
-    case method_type_ANTHY:
-      return anthy_visible();
-#endif
+    case method_type_MODULE:
+      return module_cb()->module_win_visible();
     default:
       if (!gwin_gtab)
         return FALSE;
@@ -845,9 +848,18 @@ extern int b_show_win_kbm;
 
 void hide_win_kbm();
 extern gboolean win_sym_enabled;
-int init_win_anthy();
 void show_win_kbm();
+extern char *TableDir;
 void set_gtab_input_method_name(char *s);
+GCIN_module_callback_functions *init_GCIN_module_callback_functions(char *sofile);
+time_t find_tab_file(char *fname, char *out_file);
+gboolean tsin_page_up(), tsin_page_down();
+int tsin_sele_by_idx(int);
+
+static void tsin_set_win1_cb()
+{
+  set_win1_cb((cb_selec_by_idx_t)tsin_sele_by_idx, (cb_page_ud_t)tsin_page_up, (cb_page_ud_t)tsin_page_down);
+}
 
 void update_win_kbm_inited()
 {
@@ -868,9 +880,7 @@ gboolean init_in_method(int in_no)
   if (current_CS->in_method != in_no) {
     if (!(inmd[in_no].flag & FLAG_GTAB_SYM_KBM)) {
       if (current_method_type() == method_type_TSIN) {
-#if USE_TSIN
-      flush_tsin_buffer();
-#endif
+        flush_tsin_buffer();
       } else
         output_gbuf();
 
@@ -881,36 +891,52 @@ gboolean init_in_method(int in_no)
       hide_win_kbm();
   }
 
+
   reset_current_in_win_xy();
 
 //  dbg("switch init_in_method %x %d\n", current_CS, in_no);
+  set_tsin_pho_mode0(current_CS);
+  set_wselkey(pho_selkey);
+  tsin_set_win1_cb();
 
   switch (inmd[in_no].method_type) {
     case method_type_PHO:
       current_CS->in_method = in_no;
       init_tab_pho();
       break;
-#if USE_TSIN
     case method_type_TSIN:
       current_CS->in_method = in_no;
-      set_tsin_pho_mode0(current_CS);
       init_tab_pp(init_im);
       break;
-#endif
     case method_type_INT_CODE:
       current_CS->in_method = in_no;
       init_inter_code();
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
-      if (init_win_anthy()) {
+    case method_type_MODULE:
+    {
+      GCIN_module_main_functions gmf;
+      init_GCIN_module_main_functions(&gmf);
+      if (!module_cb()) {
+        char ttt[256];
+        if (!find_tab_file(inmd[in_no].filename, ttt))
+          return FALSE;
+
+        dbg("module %s\n", ttt);
+        if (!(inmd[in_no].mod_cb_funcs = init_GCIN_module_callback_functions(ttt))) {
+          dbg("module not found\n");
+          return FALSE;
+        }
+      }
+
+      if (inmd[in_no].mod_cb_funcs->module_init_win(&gmf)) {
         current_CS->in_method = in_no;
-        show_win_anthy();
+        module_cb()->module_show_win();
       } else {
         return FALSE;
       }
+
       break;
-#endif
+    }
     default:
       init_gtab(in_no);
       if (!inmd[in_no].DefChars)
@@ -928,13 +954,19 @@ gboolean init_in_method(int in_no)
   }
 #if WIN32
   if (current_CS && current_CS->in_method != last_input_method)
-	  last_input_method = current_CS->in_method;
+    last_input_method = current_CS->in_method;
 #endif
 
 #if TRAY_ENABLED
   disp_tray_icon();
 #endif
 
+  if (inmd[current_CS->in_method].selkey) {
+    set_wselkey(inmd[current_CS->in_method].selkey);
+    gtab_set_win1_cb();
+    dbg("selkey %s\n", inmd[current_CS->in_method].selkey);
+  }
+
   update_in_win_pos();
   update_win_kbm_inited();
 
@@ -999,7 +1031,6 @@ int feedkey_pp(KeySym xkey, int state);
 int feedkey_gtab(KeySym key, int kbstate);
 int feed_phrase(KeySym ksym, int state);
 int feedkey_intcode(KeySym key);
-gboolean feedkey_anthy(int kv, int kvstate);
 void tsin_set_eng_ch(int nmod);
 static KeySym last_keysym;
 
@@ -1012,21 +1043,51 @@ gboolean timeout_raise_window(gpointer data)
 }
 
 extern Window xwin_pho, xwin0, xwin_gtab;
-void create_win_sym();
+void create_win_sym(), win_kbm_disp_caplock();
 int gcin_switch_keys_lookup(int key);
 
+void disp_win_kbm_capslock()
+{
+  if (!b_show_win_kbm)
+    return;
+
+  gboolean o_state = old_capslock_on;
+  old_capslock_on = gdk_keymap_get_caps_lock_state(gdk_keymap_get_default());
+
+//  dbg("%x %x\n", old_capslock_on, o_state);
+
+  if (o_state != old_capslock_on) {
+    win_kbm_disp_caplock();
+  }
+}
+
+
+void disp_win_kbm_capslock_init()
+{
+  old_capslock_on = gdk_keymap_get_caps_lock_state(gdk_keymap_get_default());
+//  dbg("disp_win_kbm_capslock_init %d\n",old_capslock_on);
+
+  if (b_show_win_kbm)
+    win_kbm_disp_caplock();
+}
+
 // return TRUE if the key press is processed
 gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 {
 #if 0
   dbg("key press %x %x\n", keysym, kev_state);
 #endif
+  disp_win_kbm_capslock();
   check_CS();
 
   if (current_CS->client_win)
     focus_win = current_CS->client_win;
 
-  if (!test_mode && callback_str_buffer && strlen(callback_str_buffer)) {
+  if (
+#if WIN32
+  !test_mode &&
+#endif
+  callback_str_buffer && strlen(callback_str_buffer)) {
     send_text(callback_str_buffer);
     callback_str_buffer[0]=0;
     return TRUE;
@@ -1090,7 +1151,14 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       return TRUE;
     }
 
-    if (keysym == ',') {
+    if (!gcin_enable_ctrl_alt_switch)
+      return FALSE;
+
+    int kidx = gcin_switch_keys_lookup(keysym);
+    if (kidx <= 0)
+      return FALSE;
+
+    if (inmd[kidx].method_type == method_type_SYMBOL_TABLE) {
 #if 1
       if (current_CS->im_state == GCIN_STATE_CHINESE) {
         if (!win_is_visible())
@@ -1113,13 +1181,6 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
       return TRUE;
     }
 
-    if (!gcin_enable_ctrl_alt_switch)
-      return FALSE;
-
-    int kidx = gcin_switch_keys_lookup(keysym);
-    if (kidx <= 0)
-      return FALSE;
-
     if (!inmd[kidx].cname)
       return FALSE;
 
@@ -1168,10 +1229,8 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 #endif
     case method_type_INT_CODE:
       return feedkey_intcode(keysym);
-#if USE_ANTHY
-    case method_type_ANTHY:
-      return feedkey_anthy(keysym, kev_state);
-#endif
+    case method_type_MODULE:
+      return module_cb()->module_feedkey(keysym, kev_state);
     default:
       return feedkey_gtab(keysym, kev_state);
   }
@@ -1180,12 +1239,13 @@ gboolean ProcessKeyPress(KeySym keysym, u_int kev_state)
 }
 
 int feedkey_pp_release(KeySym xkey, int kbstate);
-int feedkey_anthy_release(KeySym xkey, int kbstate);
 int feedkey_gtab_release(KeySym xkey, int kbstate);
 
 // return TRUE if the key press is processed
 gboolean ProcessKeyRelease(KeySym keysym, u_int kev_state)
 {
+  disp_win_kbm_capslock();
+
   check_CS();
 #if 0
   dbg_time("key release %x %x\n", keysym, kev_state);
@@ -1210,10 +1270,8 @@ gboolean ProcessKeyRelease(KeySym keysym, u_int kev_state)
   switch(current_method_type()) {
     case method_type_TSIN:
       return feedkey_pp_release(keysym, kev_state);
-#if USE_ANTHY
-    case method_type_ANTHY:
-      return feedkey_anthy_release(keysym, kev_state);
-#endif
+    case method_type_MODULE:
+      return module_cb()->module_feedkey_release(keysym, kev_state);
     default:
       return feedkey_gtab_release(keysym, kev_state);
   }
@@ -1303,13 +1361,18 @@ int gcin_FocusIn(ClientState *cs)
     if (cs->im_state != GCIN_STATE_DISABLED) {
       show_in_win(cs);
       move_IC_in_win(cs);
-#if 0
-      set_win_status_inmd(inmd[cs->in_method].cname);
-#endif
     } else
       hide_in_win(cs);
   }
 
+  if (inmd[cs->in_method].selkey)
+    set_wselkey(inmd[cs->in_method].selkey);
+  else {
+    set_wselkey(pho_selkey);
+    gtab_set_win1_cb();
+    tsin_set_win1_cb();
+  }
+
   update_win_kbm();
 #if TRAY_ENABLED
   disp_tray_icon();
@@ -1385,7 +1448,6 @@ int gcin_FocusOut(ClientState *cs)
 }
 
 int tsin_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
-int anthy_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor);
 int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub_comp_len);
 int int_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
 int pho_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *cursor, int *sub_comp_len);
@@ -1415,10 +1477,8 @@ empty:
     case method_type_TSIN:
       return tsin_get_preedit(str, attr, cursor, sub_comp_len);
 #endif
-#if USE_ANTHY
-    case method_type_ANTHY:
-      return anthy_get_preedit(str, attr, cursor);
-#endif
+    case method_type_MODULE:
+      return module_cb()->module_get_preedit(str, attr, cursor);
     default:
       return gtab_get_preedit(str, attr, cursor, sub_comp_len);
 //      dbg("metho %d\n", current_CS->in_method);
@@ -1428,7 +1488,7 @@ empty:
 }
 
 void pho_reset();
-int tsin_reset(),gcin_anthy_reset();
+int tsin_reset();
 void gtab_reset();
 
 void gcin_reset()
@@ -1449,11 +1509,9 @@ void gcin_reset()
       tsin_reset();
       return;
 #endif
-#if USE_ANTHY
-    case method_type_ANTHY:
-      gcin_anthy_reset();
+    case method_type_MODULE:
+      module_cb()->module_reset();
       return;
-#endif
     default:
       gtab_reset();
 //      dbg("metho %d\n", current_CS->in_method);
@@ -1485,7 +1543,6 @@ gboolean gcin_edit_display_ap_only()
   return current_CS->use_preedit && gcin_edit_display==GCIN_EDIT_DISPLAY_ON_THE_SPOT;
 }
 
-void flush_anthy_input();
 
 void flush_edit_buffer()
 {
@@ -1499,11 +1556,9 @@ void flush_edit_buffer()
       flush_tsin_buffer();
       break;
 #endif
-#if USE_ANTHY
-    case method_type_ANTHY:
-      flush_anthy_input();
+    case method_type_MODULE:
+      module_cb()->module_flush_input();
       break;
-#endif
     default:
       output_gbuf();
 //      dbg("metho %d\n", current_CS->in_method);
@@ -1542,10 +1597,8 @@ gboolean ProcessTestKeyPress(KeySym keysym, u_int kev_state)
       v = ProcessKeyPress(keysym, kev_state);
       tsin_restore_gst();
       break;
-#if USE_ANTHY
-    case method_type_ANTHY:
+    case method_type_MODULE:
       break;
-#endif
     default:
       gtab_save_gst();
       v = ProcessKeyPress(keysym, kev_state);
