@@ -2,20 +2,21 @@
 #include "gtab.h"
 
 int gcin_font_size, gcin_font_size_tsin_presel, gcin_font_size_symbol;
-int gcin_font_size_pho_near, gcin_font_size_gtab_in, gcin_win_color_use;
+int gcin_font_size_pho_near, gcin_font_size_gtab_in, gcin_font_size_win_kbm, gcin_font_size_win_kbm_en;
+int gcin_win_color_use;
 int gcin_remote_client;
 int default_input_method;
 int left_right_button_tips;
 int gcin_im_toggle_keys, gcin_bell_off;
 int gcin_capslock_lower, gcin_eng_phrase_enabled, gcin_init_im_enabled;
 int gcin_win_sym_click_close, gcin_edit_display, gcin_win32_icon;
-int gcin_on_the_spot_key;
+int gcin_on_the_spot_key, gcin_tray_hf_win_kbm;
 
 int gtab_dup_select_bell;
 int gtab_space_auto_first;
 int gtab_auto_select_by_phrase;
 int gtab_press_full_auto_send;
-int gtab_pre_select;
+int gtab_pre_select, gtab_phrase_pre_select;
 int gtab_disp_partial_match;
 int gtab_disp_key_codes;
 int gtab_disp_im_name;
@@ -57,6 +58,8 @@ int gcb_enabled, gcb_position, gcb_position_x, gcb_position_y;
 #endif
 int gcin_bell_volume;
 int gcin_sound_play_overlap, gcin_enable_ctrl_alt_switch;
+char *pho_kbm_name, *pho_selkey;
+int pho_candicate_col_N, pho_candicate_R2L;
 
 
 int get_gcin_conf_int(char *name, int default_value);
@@ -74,12 +77,15 @@ void load_setttings()
   gcin_font_size_tsin_pho_in = get_gcin_conf_int(GCIN_FONT_SIZE_TSIN_PHO_IN, 10);
   gcin_font_size_gtab_in = get_gcin_conf_int(GCIN_FONT_SIZE_GTAB_IN, 10);
   gcin_font_size_pho_near = get_gcin_conf_int(GCIN_FONT_SIZE_PHO_NEAR, 14);
+  gcin_font_size_win_kbm = get_gcin_conf_int(GCIN_FONT_SIZE_WIN_KBM, 10);
+  gcin_font_size_win_kbm_en = get_gcin_conf_int(GCIN_FONT_SIZE_WIN_KBM_EN, 8);
   gcin_input_style = get_gcin_conf_int(GCIN_INPUT_STYLE, InputStyleOverSpot);
-  gcin_root_x = get_gcin_conf_int(GCIN_ROOT_X, 2000);
-  gcin_root_y = get_gcin_conf_int(GCIN_ROOT_Y, 2000);
+  gcin_root_x = get_gcin_conf_int(GCIN_ROOT_X, 1600);
+  gcin_root_y = get_gcin_conf_int(GCIN_ROOT_Y, 1200);
   gcin_pop_up_win = get_gcin_conf_int(GCIN_POP_UP_WIN, 1);
   gcin_inner_frame = get_gcin_conf_int(GCIN_INNER_FRAME, 0);
   gcin_eng_phrase_enabled = get_gcin_conf_int(GCIN_ENG_PHRASE_ENABLED, 1);
+  gcin_tray_hf_win_kbm = get_gcin_conf_int(GCIN_TRAY_HF_WIN_KBM, 0);
 #if UNIX
   gcin_init_im_enabled = get_gcin_conf_int(GCIN_INIT_IM_ENABLED, 0);
 #else
@@ -100,8 +106,9 @@ void load_setttings()
   gcin_win_sym_click_close = get_gcin_conf_int(GCIN_WIN_SYM_CLICK_CLOSE, 1);
 #if WIN32
   gcin_win32_icon = 1;
-#else
-  gcin_win32_icon = get_gcin_conf_int(GCIN_WIN32_ICON, 0);
+#endif
+#if UNIX
+  gcin_win32_icon = get_gcin_conf_int(GCIN_WIN32_ICON, 1);
 #endif
 
   gtab_dup_select_bell = get_gcin_conf_int(GTAB_DUP_SELECT_BELL, 0);
@@ -120,6 +127,7 @@ void load_setttings()
   gtab_vertical_select = get_gcin_conf_int(GTAB_VERTICAL_SELECT, 0);
   gtab_unique_auto_send = get_gcin_conf_int(GTAB_UNIQUE_AUTO_SEND, 0);
   gtab_que_wild_card = get_gcin_conf_int(GTAB_QUE_WILD_CARD, 0);
+  gtab_phrase_pre_select = get_gcin_conf_int(GTAB_PHRASE_PRE_SELECT, 1);
 
   tsin_phrase_pre_select = get_gcin_conf_int(TSIN_PHRASE_PRE_SELECT, 1);
   tsin_chinese_english_toggle_key = get_gcin_conf_int(TSIN_CHINESE_ENGLISH_TOGGLE_KEY,
@@ -130,7 +138,7 @@ void load_setttings()
   tsin_buffer_size = get_gcin_conf_int(TSIN_BUFFER_SIZE, 40);
   tsin_tab_phrase_end = get_gcin_conf_int(TSIN_TAB_PHRASE_END, 1);
   tsin_tail_select_key = get_gcin_conf_int(TSIN_TAIL_SELECT_KEY, 0);
-  tsin_buffer_editing_mode = get_gcin_conf_int(TSIN_BUFFER_EDITING_MODE, 0);
+  tsin_buffer_editing_mode = get_gcin_conf_int(TSIN_BUFFER_EDITING_MODE, 1);
 
   phonetic_char_dynamic_sequence = get_gcin_conf_int(PHONETIC_CHAR_DYNAMIC_SEQUENCE, 1);
   phonetic_huge_tab = get_gcin_conf_int(PHONETIC_HUGE_TAB, 0);
@@ -159,9 +167,9 @@ void load_setttings()
   gcin_bell_volume = get_gcin_conf_int(GCIN_BELL_VOLUME, -97);
   gcin_sound_play_overlap = get_gcin_conf_int(GCIN_SOUND_PLAY_OVERLAP, 0);
   gcin_enable_ctrl_alt_switch = get_gcin_conf_int(GCIN_ENABLE_CTRL_ALT_SWITCH, 1);
-#if 0
+#if 1
   gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_BOTH);
-#elif 1
+#elif 0
   gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_ON_THE_SPOT);
 #else
   gcin_edit_display = get_gcin_conf_int(GCIN_EDIT_DISPLAY, GCIN_EDIT_DISPLAY_OVER_THE_SPOT);
@@ -170,4 +178,37 @@ void load_setttings()
   gcin_on_the_spot_key = get_gcin_conf_int(GCIN_ON_THE_SPOT_KEY, 0);
   if (gcin_on_the_spot_key)
     gcin_edit_display = GCIN_EDIT_DISPLAY_ON_THE_SPOT;
+
+
+  char phokbm[MAX_GCIN_STR];
+#define ASDF "asdfghjkl;"
+
+#if DEBUG
+  char *gcin_pho_kbm = getenv("GCIN_PHO_KBM");
+  if (gcin_pho_kbm)
+    strcpy(phokbm, gcin_pho_kbm);
+  else
+#endif
+    get_gcin_conf_fstr(PHONETIC_KEYBOARD, phokbm, "zo "ASDF" 1 1");
+
+  char phokbm_name[32], selkey[32];
+  pho_candicate_col_N=0; pho_candicate_R2L=0;
+  sscanf(phokbm, "%s %s %d %d",phokbm_name, selkey, &pho_candicate_col_N, &pho_candicate_R2L);
+
+  if (pho_candicate_col_N <= 0)
+    pho_candicate_col_N = 1;
+  if (pho_candicate_col_N > strlen(selkey))
+    pho_candicate_col_N =strlen(selkey);
+
+  if (pho_candicate_R2L<0 || pho_candicate_R2L>1)
+    pho_candicate_R2L = 0;
+
+  if (pho_selkey)
+    free(pho_selkey);
+
+  pho_selkey = strdup(selkey);
+
+  if (pho_kbm_name)
+    free(pho_kbm_name);
+  pho_kbm_name = strdup(phokbm_name);
 }
