@@ -16,7 +16,9 @@ static GtkWidget *check_button_gtab_dup_select_bell,
                  *check_button_gtab_vertical_select,
                  *check_button_gtab_unique_auto_send,
                  *check_button_gtab_que_wild_card,
-                 *check_button_gcin_capslock_lower;
+                 *check_button_gtab_phrase_pre_select;
+
+extern GtkWidget *check_button_gcin_capslock_lower;
 
 struct {
   unich_t *str;
@@ -40,12 +42,14 @@ struct {
   { NULL, 0},
 };
 
+void save_tsin_eng_pho_key();
 static GtkWidget *gcin_gtab_conf_window;
 static GtkWidget *opt_spc_opts, *opt_auto_select_by_phrase;
 static gboolean cb_gtab_conf_ok( GtkWidget *widget,
                                    GdkEvent  *event,
                                    gpointer   data )
 {
+  save_tsin_eng_pho_key();
   save_gcin_conf_int(GTAB_DUP_SELECT_BELL,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_dup_select_bell)));
 
@@ -92,22 +96,21 @@ static gboolean cb_gtab_conf_ok( GtkWidget *widget,
   save_gcin_conf_int(GCIN_CAPSLOCK_LOWER,
     gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower)));
 
-#if GTK_CHECK_VERSION(2,4,0)
+  save_gcin_conf_int(GTAB_PHRASE_PRE_SELECT,
+    gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gtab_phrase_pre_select)));
+
   int idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_spc_opts));
-#else
-  int idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_spc_opts));
-#endif
   save_gcin_conf_int(GTAB_SPACE_AUTO_FIRST, spc_opts[idx].num);
 
 
-#if GTK_CHECK_VERSION(2,4,0)
   idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_auto_select_by_phrase));
-#else
-  idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_auto_select_by_phrase));
-#endif
   save_gcin_conf_int(GTAB_AUTO_SELECT_BY_PHRASE, auto_select_by_phrase_opts[idx].num);
 
-  send_gcin_message(GDK_DISPLAY(), CHANGE_FONT_SIZE);
+  send_gcin_message(
+#if UNIX
+	  GDK_DISPLAY(),
+#endif
+	  CHANGE_FONT_SIZE);
   gtk_widget_destroy(gcin_gtab_conf_window); gcin_gtab_conf_window = NULL;
 
   return TRUE;
@@ -127,7 +130,7 @@ static gboolean cb_gtab_edit_append( GtkWidget *widget,
                                    GdkEvent  *event,
                                    gpointer   data )
 {
-  load_gtab_list();
+  load_gtab_list(FALSE);
   char *fname = inmd[default_input_method].filename;
   if (!fname)
     return TRUE;
@@ -149,7 +152,11 @@ static gboolean cb_gtab_edit_append( GtkWidget *widget,
   strcat(strcpy(append_fname, fname), ".append");
   char user_fname[512];
   get_gcin_user_fname(append_fname, user_fname);
-  win32exec_script("gtab.append_prepare.bat", user_fname);
+  if (GetFileAttributesA(user_fname) == INVALID_FILE_ATTRIBUTES)
+    win32exec_script("gtab.append_prepare.bat", user_fname);
+  else {
+    win32exec_script("utf8-edit.bat", user_fname);
+  }
 #endif
   return TRUE;
 }
@@ -160,10 +167,8 @@ static GtkWidget *create_spc_opts()
   GtkWidget *label = gtk_label_new(_(_L("空白鍵選項")));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_spc_opts = gtk_combo_box_new_text ();
-#else
-  opt_spc_opts = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu_spc_opts = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_spc_opts, FALSE, FALSE, 0);
@@ -179,18 +184,16 @@ static GtkWidget *create_spc_opts()
       current_idx = i;
 
 #if GTK_CHECK_VERSION(2,4,0)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (opt_spc_opts), _(spc_opts[i].str));
+    gtk_combo_box_append_text (GTK_COMBO_BOX_TEXT (opt_spc_opts), _(spc_opts[i].str));
 #else
     gtk_menu_shell_append (GTK_MENU_SHELL (menu_spc_opts), item);
 #endif
   }
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_spc_opts), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_spc_opts), menu_spc_opts);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_spc_opts), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_spc_opts), current_idx);
 
   return hbox;
 }
@@ -199,10 +202,8 @@ static GtkWidget *create_auto_select_by_phrase_opts()
 {
   GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_auto_select_by_phrase = gtk_combo_box_new_text ();
-#else
-  opt_auto_select_by_phrase = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu_auto_select_by_phrase = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_auto_select_by_phrase, FALSE, FALSE, 0);
@@ -218,22 +219,23 @@ static GtkWidget *create_auto_select_by_phrase_opts()
       current_idx = i;
 
 #if GTK_CHECK_VERSION(2,4,0)
-    gtk_combo_box_append_text (GTK_COMBO_BOX (opt_auto_select_by_phrase), _(auto_select_by_phrase_opts[i].str));
+    gtk_combo_box_append_text (GTK_COMBO_BOX_TEXT (opt_auto_select_by_phrase), _(auto_select_by_phrase_opts[i].str));
 #else
     gtk_menu_shell_append (GTK_MENU_SHELL (menu_auto_select_by_phrase), item);
 #endif
   }
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_auto_select_by_phrase), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_auto_select_by_phrase), menu_auto_select_by_phrase);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_auto_select_by_phrase), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_auto_select_by_phrase), current_idx);
 
   return hbox;
 }
 
+
+GtkWidget *create_en_pho_key_sel(char *s);
+
 void create_gtab_conf_window()
 {
   if (gcin_gtab_conf_window) {
@@ -244,6 +246,8 @@ void create_gtab_conf_window()
   load_setttings();
 
   gcin_gtab_conf_window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_position(GTK_WINDOW(gcin_gtab_conf_window), GTK_WIN_POS_MOUSE);
+  gtk_window_set_has_resize_grip(GTK_WINDOW(gcin_gtab_conf_window), FALSE);
 
   g_signal_connect (G_OBJECT (gcin_gtab_conf_window), "delete_event",
                     G_CALLBACK (close_gtab_conf_window),
@@ -360,6 +364,12 @@ void create_gtab_conf_window()
   GtkWidget *label_gtab_auto_select = gtk_label_new(_(_L("由詞庫自動選擇字")));
   gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase), label_gtab_auto_select,  FALSE, FALSE, 0);
   gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase), create_auto_select_by_phrase_opts(),  FALSE, FALSE, 0);
+  GtkWidget *label_gtab_phrase_pre_select = gtk_label_new(_(_L("使用預選詞")));
+  gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase), label_gtab_phrase_pre_select,  FALSE, FALSE, 0);
+  check_button_gtab_phrase_pre_select = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_gtab_auto_select_by_phrase), check_button_gtab_phrase_pre_select,  FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_phrase_pre_select), gtab_phrase_pre_select);
+
 
   GtkWidget *hbox_gtab_dup_select_bell = gtk_hbox_new (FALSE, SPC);
   gtk_box_pack_start (GTK_BOX (vbox_gtab_r), hbox_gtab_dup_select_bell, FALSE, FALSE, 0);
@@ -392,16 +402,9 @@ void create_gtab_conf_window()
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_shift_phrase_key),
      gtab_shift_phrase_key);
 
+  gtk_box_pack_start (GTK_BOX (vbox_gtab_r), create_en_pho_key_sel(_(_L("[中/英]切換"))), FALSE, FALSE, 0);
 
-  GtkWidget *hbox_gtab_capslock_in_eng = gtk_hbox_new (FALSE, SPC);
-  gtk_box_pack_start (GTK_BOX (vbox_gtab_r), hbox_gtab_capslock_in_eng, FALSE, FALSE, 0);
-  GtkWidget *label_gtab_capslock_in_eng = gtk_label_new(_(_L("CapsLock 打開輸入英數")));
-  gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), label_gtab_capslock_in_eng,  FALSE, FALSE, 0);
-  check_button_gtab_capslock_in_eng = gtk_check_button_new ();
-  gtk_box_pack_start (GTK_BOX (hbox_gtab_capslock_in_eng), check_button_gtab_capslock_in_eng,  FALSE, FALSE, 0);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gtab_capslock_in_eng),
-     gtab_capslock_in_eng);
-
+#if 0
   GtkWidget *hbox_gcin_capslock_lower = gtk_hbox_new (FALSE, SPC);
   gtk_box_pack_start (GTK_BOX (vbox_gtab_r), hbox_gcin_capslock_lower, FALSE, FALSE, 0);
   GtkWidget *label_gcin_capslock_lower = gtk_label_new(_(_L("\t用小寫字母")));
@@ -410,7 +413,7 @@ void create_gtab_conf_window()
   gtk_box_pack_start (GTK_BOX (hbox_gcin_capslock_lower), check_button_gcin_capslock_lower,  FALSE, FALSE, 0);
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower),
      gcin_capslock_lower);
-
+#endif
 
   GtkWidget *hbox_gtab_unique_auto_send = gtk_hbox_new (FALSE, SPC);
   gtk_box_pack_start (GTK_BOX (vbox_gtab_r), hbox_gtab_unique_auto_send, FALSE, FALSE, 0);
