@@ -20,10 +20,11 @@ static GtkWidget *check_button_tsin_phrase_pre_select,
                  *check_button_tsin_tab_phrase_end,
                  *check_button_tsin_tail_select_key,
                  *check_button_tsin_buffer_editing_mode,
-                 *check_button_gcin_capslock_lower,
                  *spinner_tsin_buffer_size,
                  *spinner_pho_candicate_col_N;
 
+GtkWidget *check_button_gcin_capslock_lower;
+
 static GtkWidget *opt_kbm_opts, *opt_selkeys, *opt_eng_ch_opts;
 
 
@@ -32,10 +33,10 @@ static struct {
   int key;
 } tsin_eng_ch_sw[]={
   {N_(_L("CapsLock")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock},
-  {N_(_L("Tab")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab},
-  {N_(_L("Shift(限非 XIM)")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift},
-  {N_(_L("ShiftL(限非 XIM)")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL},
-  {N_(_L("ShiftR(限非 XIM)")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR},
+//  {N_(_L("Tab")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Tab},
+  {N_(_L("Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift},
+  {N_(_L("左Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL},
+  {N_(_L("右Shift")), TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR},
 };
 int tsin_eng_ch_swN = sizeof(tsin_eng_ch_sw) / sizeof(tsin_eng_ch_sw[0]);
 
@@ -63,6 +64,17 @@ int get_current_speaker_idx()
   return 0;
 }
 
+void save_tsin_eng_pho_key()
+{
+  int idx;
+  idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_eng_ch_opts));
+  save_gcin_conf_int(TSIN_CHINESE_ENGLISH_TOGGLE_KEY,
+                     tsin_eng_ch_sw[idx].key);
+
+  save_gcin_conf_int(GCIN_CAPSLOCK_LOWER,
+       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower)));
+}
+
 
 static GtkWidget *gcin_kbm_window = NULL;
 
@@ -74,17 +86,9 @@ static gboolean cb_ok( GtkWidget *widget,
                                    GdkEvent  *event,
                                    gpointer   data )
 {
-#if GTK_CHECK_VERSION(2,4,0)
   int idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_kbm_opts));
-#else
-  int idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_kbm_opts));
-#endif
 
-#if GTK_CHECK_VERSION(2,4,0)
   int idx_selkeys = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_selkeys));
-#else
-  int idx_selkeys = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_selkeys));
-#endif
 
   pho_candicate_col_N = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_pho_candicate_col_N));
 
@@ -104,13 +108,7 @@ static gboolean cb_ok( GtkWidget *widget,
   }
   save_gcin_conf_str(PHONETIC_KEYBOARD, tt);
 
-#if GTK_CHECK_VERSION(2,4,0)
-  idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_eng_ch_opts));
-#else
-  idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_eng_ch_opts));
-#endif
-  save_gcin_conf_int(TSIN_CHINESE_ENGLISH_TOGGLE_KEY,
-                     tsin_eng_ch_sw[idx].key);
+  save_tsin_eng_pho_key();
 
   save_gcin_conf_int(TSIN_SPACE_OPT,
                      tsin_space_options[new_select_idx_tsin_space_opt].key);
@@ -143,9 +141,6 @@ static gboolean cb_ok( GtkWidget *widget,
   save_gcin_conf_int(TSIN_BUFFER_EDITING_MODE,
        gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_tsin_buffer_editing_mode)));
 
-  save_gcin_conf_int(GCIN_CAPSLOCK_LOWER,
-       gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower)));
-
   tsin_buffer_size = (int) gtk_spin_button_get_value(GTK_SPIN_BUTTON(spinner_tsin_buffer_size));
   save_gcin_conf_int(TSIN_BUFFER_SIZE, tsin_buffer_size);
 
@@ -330,10 +325,8 @@ static GtkWidget *create_kbm_opts()
 
   GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_kbm_opts = gtk_combo_box_new_text ();
-#else
-  opt_kbm_opts = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu_kbm_opts = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_kbm_opts, FALSE, FALSE, 0);
@@ -346,22 +339,17 @@ static GtkWidget *create_kbm_opts()
     gtk_combo_box_append_text (GTK_COMBO_BOX_TEXT (opt_kbm_opts), _(kbm_sel[i].name));
 #else
     GtkWidget *item = gtk_menu_item_new_with_label (_(kbm_sel[i].name));
-
     gtk_menu_shell_append (GTK_MENU_SHELL (menu_kbm_opts), item);
 #endif
   }
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_kbm_opts), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_kbm_opts), menu_kbm_opts);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_kbm_opts), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_kbm_opts), current_idx);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_selkeys = gtk_combo_box_new_text ();
-#else
-  opt_selkeys = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu_selkey = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_selkeys, FALSE, FALSE, 0);
@@ -379,12 +367,10 @@ static GtkWidget *create_kbm_opts()
       current_idx = i;
   }
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_selkeys), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_selkeys), menu_kbm_opts);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_selkeys), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_selkeys), current_idx);
 
   GtkAdjustment *adj =
    (GtkAdjustment *) gtk_adjustment_new (pho_candicate_col_N, 1, 10, 1.0, 1.0, 0.0);
@@ -401,10 +387,8 @@ static GtkWidget *create_eng_ch_opts()
 
   GtkWidget *hbox = gtk_hbox_new (FALSE, 1);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_eng_ch_opts = gtk_combo_box_new_text ();
-#else
-  opt_eng_ch_opts = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu_eng_ch_opts = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_eng_ch_opts, FALSE, FALSE, 0);
@@ -417,23 +401,38 @@ static GtkWidget *create_eng_ch_opts()
     gtk_combo_box_append_text (GTK_COMBO_BOX_TEXT (opt_eng_ch_opts), _(tsin_eng_ch_sw[i].name));
 #else
     GtkWidget *item = gtk_menu_item_new_with_label (_(tsin_eng_ch_sw[i].name));
-
     gtk_menu_shell_append (GTK_MENU_SHELL (menu_eng_ch_opts), item);
 #endif
   }
 
   dbg("current_idx:%d\n", current_idx);
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_eng_ch_opts), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_eng_ch_opts), menu_eng_ch_opts);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_eng_ch_opts), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_eng_ch_opts), current_idx);
 
   return hbox;
 }
 
+GtkWidget *create_en_pho_key_sel(char *s)
+{
+GtkWidget *frame_tsin_sw = gtk_frame_new(s);
+  GtkWidget *vbox_tsin_sw = gtk_vbox_new(FALSE, 0);
+  gtk_container_add (GTK_CONTAINER (frame_tsin_sw), vbox_tsin_sw);
+  gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_sw), 1);
+  gtk_container_add (GTK_CONTAINER (vbox_tsin_sw), create_eng_ch_opts());
+  GtkWidget *hbox_gcin_capslock_lower = gtk_hbox_new(FALSE, 0);
+  gtk_box_pack_start (GTK_BOX (vbox_tsin_sw), hbox_gcin_capslock_lower, TRUE, TRUE, 0);
+  GtkWidget *label_gcin_capslock_lower = gtk_label_new(_(_L("Capslock英數用小寫")));
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_capslock_lower), label_gcin_capslock_lower , TRUE, TRUE, 0);
+  check_button_gcin_capslock_lower = gtk_check_button_new ();
+  gtk_box_pack_start (GTK_BOX (hbox_gcin_capslock_lower), check_button_gcin_capslock_lower, FALSE, FALSE, 0);
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower), gcin_capslock_lower);
+
+  return frame_tsin_sw;
+}
+
 
 void load_setttings();
 
@@ -477,24 +476,7 @@ void create_kbm_window()
   gtk_container_set_border_width (GTK_CONTAINER (frame_kbm), 1);
   gtk_container_add (GTK_CONTAINER (frame_kbm), create_kbm_opts());
 
-
-  GtkWidget *frame_tsin_sw = gtk_frame_new(_(_L("詞音輸入[中/英]切換")));
-  gtk_box_pack_start (GTK_BOX (vbox_l), frame_tsin_sw, TRUE, TRUE, 0);
-  GtkWidget *vbox_tsin_sw = gtk_vbox_new(FALSE, 0);
-  gtk_container_add (GTK_CONTAINER (frame_tsin_sw), vbox_tsin_sw);
-  gtk_container_set_border_width (GTK_CONTAINER (frame_tsin_sw), 1);
-  gtk_container_add (GTK_CONTAINER (vbox_tsin_sw), create_eng_ch_opts());
-
-  GtkWidget *hbox_gcin_capslock_lower = gtk_hbox_new(FALSE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox_tsin_sw), hbox_gcin_capslock_lower, TRUE, TRUE, 0);
-  gtk_box_pack_start (GTK_BOX (vbox_r), hbox_gcin_capslock_lower , TRUE, TRUE, 1);
-  GtkWidget *label_gcin_capslock_lower = gtk_label_new(_(_L("Capslock英數用小寫")));
-  gtk_box_pack_start (GTK_BOX (hbox_gcin_capslock_lower), label_gcin_capslock_lower , TRUE, TRUE, 0);
-  check_button_gcin_capslock_lower = gtk_check_button_new ();
-  gtk_box_pack_start (GTK_BOX (hbox_gcin_capslock_lower), check_button_gcin_capslock_lower, FALSE, FALSE, 0);
-  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_capslock_lower), gcin_capslock_lower);
-
-
+  gtk_box_pack_start (GTK_BOX (vbox_l), create_en_pho_key_sel(_(_L("詞音輸入[中/英]切換"))), TRUE, TRUE, 0);
 
   GtkWidget *frame_tsin_space_opt = gtk_frame_new(_(_L("詞音輸入空白鍵選項")));
   gtk_box_pack_start (GTK_BOX (vbox_l), frame_tsin_space_opt, TRUE, TRUE, 0);
