@@ -13,9 +13,8 @@ static GtkWidget *check_button_root_style_use,
                  *check_button_gcin_inner_frame,
 #if TRAY_ENABLED
                  *check_button_gcin_status_tray,
-#if !GTK_CHECK_VERSION(2,91,0)
                  *check_button_gcin_win32_icon,
-#endif
+		     *check_button_gcin_tray_hf_win_kbm,
 #endif
                  *check_button_gcin_win_color_use,
                  *check_button_gcin_on_the_spot_key;
@@ -389,11 +388,7 @@ static gboolean cb_appearance_conf_ok( GtkWidget *widget,
   save_gcin_conf_int(GCIN_INNER_FRAME, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_inner_frame)));
 #if TRAY_ENABLED
   save_gcin_conf_int(GCIN_STATUS_TRAY, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray)));
-#if UNIX && GTK_CHECK_VERSION(2,91,0)
-  save_gcin_conf_int(GCIN_WIN32_ICON, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray)));
-#else
   save_gcin_conf_int(GCIN_WIN32_ICON, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_win32_icon)));
-#endif
 #endif
 
   gchar *cstr = gtk_color_selection_palette_to_string(&gcin_win_gcolor_fg, 1);
@@ -408,16 +403,13 @@ static gboolean cb_appearance_conf_ok( GtkWidget *widget,
 
   save_gcin_conf_int(GCIN_WIN_COLOR_USE, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_win_color_use)));
   save_gcin_conf_int(GCIN_ON_THE_SPOT_KEY, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_on_the_spot_key)));
+  save_gcin_conf_int(GCIN_TRAY_HF_WIN_KBM, gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(check_button_gcin_tray_hf_win_kbm)));
 
   cstr = gtk_color_selection_palette_to_string(&gcin_sel_key_gcolor, 1);
   dbg("selkey color %s\n", cstr);
   save_gcin_conf_str(GCIN_SEL_KEY_COLOR, cstr);
 
-#if GTK_CHECK_VERSION(2,4,0)
   int idx = gtk_combo_box_get_active (GTK_COMBO_BOX (opt_gcin_edit_display));
-#else
-  int idx = gtk_option_menu_get_history (GTK_OPTION_MENU (opt_gcin_edit_display));
-#endif
   save_gcin_conf_int(GCIN_EDIT_DISPLAY, edit_disp[idx].keynum);
 
   g_free(cstr);
@@ -538,13 +530,7 @@ void cb_button_gcin_on_the_spot_key(GtkToggleButton *togglebutton, gpointer user
   int i;
   for (i=0; edit_disp[i].keystr; i++)
    if (edit_disp[i].keynum == GCIN_EDIT_DISPLAY_ON_THE_SPOT)
-   {
-#if GTK_CHECK_VERSION(2,4,0)
      gtk_combo_box_set_active (GTK_COMBO_BOX (opt_gcin_edit_display), i);
-#else
-     gtk_option_menu_set_history(GTK_OPTION_MENU(opt_gcin_edit_display), i);
-#endif
-   }
 }
 
 static GtkWidget *create_gcin_edit_display()
@@ -554,10 +540,8 @@ static GtkWidget *create_gcin_edit_display()
   GtkWidget *label = gtk_label_new(_(_L("編輯區顯示")));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
 
-#if GTK_CHECK_VERSION(2,4,0)
   opt_gcin_edit_display = gtk_combo_box_new_text ();
-#else
-  opt_gcin_edit_display = gtk_option_menu_new ();
+#if !GTK_CHECK_VERSION(2,4,0)
   GtkWidget *menu = gtk_menu_new ();
 #endif
   gtk_box_pack_start (GTK_BOX (hbox), opt_gcin_edit_display, FALSE, FALSE, 0);
@@ -579,12 +563,10 @@ static GtkWidget *create_gcin_edit_display()
 #endif
   }
 
-#if GTK_CHECK_VERSION(2,4,0)
-  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_gcin_edit_display), current_idx);
-#else
+#if !GTK_CHECK_VERSION(2,4,0)
   gtk_option_menu_set_menu (GTK_OPTION_MENU (opt_gcin_edit_display), menu);
-  gtk_option_menu_set_history (GTK_OPTION_MENU (opt_gcin_edit_display), current_idx);
 #endif
+  gtk_combo_box_set_active (GTK_COMBO_BOX (opt_gcin_edit_display), current_idx);
 
   label = gtk_label_new(_(_L("按鍵顯示於\n應用程式")));
   gtk_box_pack_start (GTK_BOX (hbox), label, FALSE, FALSE, 0);
@@ -775,14 +757,20 @@ void create_appearance_conf_window()
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_status_tray),
        gcin_status_tray);
   gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), check_button_gcin_status_tray, FALSE, FALSE, 0);
-#if UNIX && !GTK_CHECK_VERSION(2,91,0)
-  GtkWidget *label_gcin_status_tray_windows_style = gtk_label_new(_(_L("Windows style")));
+#if UNIX
+  GtkWidget *label_gcin_status_tray_windows_style = gtk_label_new(_(_L("雙圖示")));
   gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), label_gcin_status_tray_windows_style, FALSE, FALSE, 0);
   check_button_gcin_win32_icon = gtk_check_button_new ();
   gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_win32_icon),
        gcin_win32_icon);
   gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), check_button_gcin_win32_icon, FALSE, FALSE, 0);
 #endif
+  GtkWidget *label_gcin_tray_hf_win_kbm = gtk_label_new(_(_L("全半形左鍵\n切換小鍵盤")));
+  gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), label_gcin_tray_hf_win_kbm, FALSE, FALSE, 0);
+  check_button_gcin_tray_hf_win_kbm = gtk_check_button_new ();
+  gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(check_button_gcin_tray_hf_win_kbm),
+       gcin_tray_hf_win_kbm);
+  gtk_box_pack_start (GTK_BOX(hbox_gcin_status_tray), check_button_gcin_tray_hf_win_kbm, FALSE, FALSE, 0);
 #endif
 
   GtkWidget *frame_win_color = gtk_frame_new(_(_L("顏色選擇")));
@@ -848,11 +836,7 @@ void create_appearance_conf_window()
                             G_CALLBACK (cb_appearance_conf_ok),
                             G_OBJECT (gcin_kbm_window));
 
-#if GTK_CHECK_VERSION(2,17,5)
-  gtk_widget_set_can_default (button_close, TRUE);
-#else
   GTK_WIDGET_SET_FLAGS (button_close, GTK_CAN_DEFAULT);
-#endif
   gtk_widget_grab_default (button_close);
 
   gtk_widget_show_all (gcin_appearance_conf_window);
@@ -907,17 +891,17 @@ static void cb_juying_learn()
   exit(0);
 }
 
-
+#if 0
 int gcin_pid;
-
 static void cb_gcin_exit()
 {
 #if UNIX
   kill(gcin_pid, 9);
 #else
+  TerminateProcess(
 #endif
 }
-
+#endif
 
 void create_gtablist_window();
 static void cb_default_input_method()
@@ -977,10 +961,12 @@ static void create_main_win()
   g_signal_connect (G_OBJECT (button_symbol_table), "clicked",
                     G_CALLBACK (cb_symbol_table), NULL);
 
+#if 0
   GtkWidget *button_gb_output_toggle = gtk_button_new_with_label(_(_L("簡體字輸出切換")));
   gtk_box_pack_start (GTK_BOX (vbox), button_gb_output_toggle, TRUE, TRUE, 0);
   g_signal_connect (G_OBJECT (button_gb_output_toggle), "clicked",
                     G_CALLBACK (cb_gb_output_toggle), NULL);
+#endif
 
   GtkWidget *button_gb_translate_toggle = gtk_button_new_with_label(_(_L("剪貼區 簡體字->正體字")));
   gtk_box_pack_start (GTK_BOX (vbox), button_gb_translate_toggle, TRUE, TRUE, 0);
@@ -1040,14 +1026,15 @@ static void create_main_win()
   g_signal_connect (G_OBJECT (button_help), "clicked",
                     G_CALLBACK (cb_help), NULL);
 
+#if 0
   char *pid = getenv("GCIN_PID");
-
   if (pid && (gcin_pid = atoi(pid))) {
     GtkWidget *button_gcin_exit = gtk_button_new_with_label (_(_L("結束 gcin")));
     gtk_box_pack_start (GTK_BOX (vbox), button_gcin_exit, TRUE, TRUE, 0);
     g_signal_connect (G_OBJECT (button_gcin_exit), "clicked",
                       G_CALLBACK (cb_gcin_exit), NULL);
   }
+#endif
 
 
   GtkWidget *button_quit = gtk_button_new_from_stock (GTK_STOCK_QUIT);
