@@ -2,7 +2,7 @@
 #include "gtab.h"
 
 // space + \0'
-char gcin_switch_keys[MAX_GTAB_NUM_KEY+2]=" 1234567890-=`[]\\";
+char gcin_switch_keys[MAX_GTAB_NUM_KEY+2]=" 1234567890-=`[]\\,";
 
 int gcin_switch_keys_lookup(int key)
 {
