@@ -25,7 +25,7 @@ void lookup_gtabn(char *ch, char *out);
 char *htmlspecialchars(char *s, char out[]);
 void hide_gtab_pre_sel();
 
-extern gint64 key_press_time_ctrl;
+extern gint64 key_press_time, key_press_time_ctrl;
 
 extern gboolean test_mode;
 
@@ -866,7 +866,7 @@ int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub
   str[0]=0;
   *pcursor=0;
 
-#if WIN32
+#if WIN32 || 1
   *sub_comp_len = ggg.ci > 0;
 #if 1
   if (ggg.gbufN && !gcin_edit_display_ap_only())
@@ -903,7 +903,7 @@ int gtab_get_preedit(char *str, GCIN_PREEDIT_ATTR attr[], int *pcursor, int *sub
       ch_N+=N;
       if (i < ggg.gbuf_cursor)
         *pcursor+=N;
-      if (i==ggg.gbuf_cursor) {
+      if (ap_only && i==ggg.gbuf_cursor) {
         attr[1].ofs0=*pcursor;
         attr[1].ofs1=*pcursor+N;
         attr[1].flag=GCIN_PREEDIT_ATTR_FLAG_REVERSE;
@@ -942,16 +942,8 @@ void gtab_reset()
   return;
 }
 
-
-void save_gtab_buf_phrase(KeySym key)
+void save_gtab_buf_phrase_idx(int idx0, int len)
 {
-  int len = key - '0';
-  int idx0 = ggg.gbuf_cursor - len;
-  int idx1 = ggg.gbuf_cursor - 1;
-
-  if (idx0 < 0 || idx0 > idx1)
-    return;
-
   WSP_S wsp[MAX_PHRASE_LEN];
 
   bzero(wsp, sizeof(wsp));
@@ -964,18 +956,25 @@ void save_gtab_buf_phrase(KeySym key)
   create_win_save_phrase(wsp, len);
 }
 
+void save_gtab_buf_phrase(KeySym key)
+{
+  int len = key - '0';
+  int idx0 = ggg.gbuf_cursor - len;
+  int idx1 = ggg.gbuf_cursor - 1;
+
+  if (idx0 < 0 || idx0 > idx1)
+    return;
+
+  save_gtab_buf_phrase_idx(idx0, len);
+}
+
 gboolean save_gtab_buf_shift_enter()
 {
-	if (ggg.gbuf_cursor== ggg.gbufN)
+	int N = ggg.gbufN - ggg.gbuf_cursor;
+	if (!N)
 		return 0;
 
-	int keys[512];
-	int N = ggg.gbufN - ggg.gbuf_cursor;
-	extract_gtab_key(ggg.gbuf_cursor, N, keys);
-	char *str = gen_buf_str(ggg.gbuf_cursor, FALSE);
-	save_phrase_to_db(keys, str, N, 1);
-
-	free(str);
+	save_gtab_buf_phrase_idx(ggg.gbuf_cursor, N);
 	gbuf_cursor_end();
 	return 1;
 }
@@ -1038,6 +1037,12 @@ void init_tsin_table()
 
 extern u_char scanphr_e(int chpho_idx, int plen, gboolean pho_incr, int *rselN);
 void init_pre_sel();
+void clear_sele();
+void set_sele_text(int tN, int i, char *text, int len);
+void get_win_gtab_geom();
+void disp_selections(int x, int y);
+
+gboolean use_tsin_sel_win();
 
 static int gtab_pre_select_phrase_len;
 
@@ -1097,6 +1102,16 @@ void gtab_scan_pre_select(gboolean b_incr)
 
 //  dbg("selN %d %d\n",selN, tss.pre_selN);
 
+  if (use_tsin_sel_win()) {
+    clear_sele();
+    int i;
+    for(i=0;i<tss.pre_selN; i++)
+       set_sele_text(tss.pre_selN,i,tss.pre_sel[i].str, -1);
+    get_win_gtab_geom();
+    disp_selections(-1, -1);
+    return;
+  }
+
   char tt[4096];
   tt[0]=0;
   int i;
@@ -1143,7 +1158,6 @@ gboolean gtab_pre_select_idx(int c)
   return TRUE;
 }
 
-
 gboolean gtab_pre_select_shift(KeySym key, int kbstate)
 {
 //  dbg("gtab_pre_select_shift %c\n", key);
@@ -1154,6 +1168,7 @@ gboolean gtab_pre_select_shift(KeySym key, int kbstate)
   return gtab_pre_select_idx(c);
 }
 
+void tsin_toggle_eng_ch();
 
 int feedkey_gtab_release(KeySym xkey, int kbstate)
 {
@@ -1171,8 +1186,35 @@ int feedkey_gtab_release(KeySym xkey, int kbstate)
           return 1;
         } else
           return 0;
+#if 1
+     case XK_Shift_L:
+     case XK_Shift_R:
+		kpt = key_press_time;
+		key_press_time = 0;
+
+// dbg("release xkey %x\n", xkey);
+        if (
+(  (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_Shift) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftL
+     && xkey == XK_Shift_L) ||
+   (tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_ShiftR
+     && xkey == XK_Shift_R))
+          &&  current_time() - kpt < 300000) {
+          if (!test_mode) {
+			  tsin_toggle_eng_ch();
+          }
+          return 1;
+		} else
+          return 0;
+#endif
      default:
         return 0;
   }
 }
 
+#include "win1.h"
+
+void gtab_set_win1_cb()
+{
+  set_win1_cb((cb_selec_by_idx_t)gtab_pre_select_idx, NULL, NULL);
+}
