@@ -20,7 +20,8 @@ GTAB_ST ggg = {.sel1st_i=MAX_SELKEY - 1};
 GTAB_ST ggg;
 #endif
 
-extern gint64 key_press_time_ctrl;
+extern gint64 key_press_time, key_press_time_ctrl;
+
 extern GtkWidget *gwin_gtab;
 void hide_gtab_pre_sel();
 void gtab_scan_pre_select(gboolean);
@@ -40,6 +41,14 @@ static char keyrow[]=
 	      "asdfghjkl;"
 	      "zxcvbnm,./";
 
+gboolean use_tsin_sel_win()
+{
+  return
+//  !gtab_pre_select && !gtab_disp_partial_match &&
+  gtab_vertical_select && gtab_phrase_pre_select;
+}
+
+
 int key_col(char cha)
 {
   char *p = strchr(keyrow, cha);
@@ -460,6 +469,24 @@ void init_seltab(char ***p)
   }
 }
 
+time_t find_tab_file(char *fname, char *out_file)
+{
+  get_gcin_user_fname(fname, out_file);
+  time_t mtime = file_mtime(out_file);
+
+  if (!mtime) {
+    strcat(strcpy(out_file,TableDir),"/");
+    strcat(out_file, fname);
+    if (!(mtime = file_mtime(out_file))) {
+      dbg("init_tab:1 err open %s\n", out_file);
+      return 0;
+    }
+  }
+
+  return mtime;
+}
+
+
 void init_gtab(int inmdno)
 {
   FILE *fp;
@@ -477,17 +504,11 @@ void init_gtab(int inmdno)
     return;
   }
 
-  get_gcin_user_fname(inmd[inmdno].filename, ttt);
-  time_t mtime = file_mtime(ttt);
+  time_t mtime;
+
+  if (!(mtime = find_tab_file(inmd[inmdno].filename, ttt)))
+    return;
 
-  if (!mtime) {
-    strcat(strcpy(ttt,TableDir),"/");
-    strcat(ttt, inmd[inmdno].filename);
-    if (!(mtime = file_mtime(ttt))) {
-      dbg("init_tab:1 err open %s\n", ttt);
-      return;
-    }
-  }
 
   char append[64], append_user[128];
   strcat(strcpy(append, inmd[inmdno].filename), ".append");
@@ -649,7 +670,18 @@ void init_gtab(int inmdno)
   inp->MaxPress=th.MaxPress;
   inp->DefChars=th.DefC;
   free(inp->selkey);
-  inp->selkey = strdup(th.selkey);
+
+  if (th.selkey[sizeof(th.selkey)-1]) {
+    char tt[32];
+    bzero(tt, sizeof(tt));
+    memcpy(tt,th.selkey, sizeof(th.selkey));
+    strcat(tt, th.selkey2);
+    inp->selkey = strdup(tt);
+  } else
+    inp->selkey = strdup(th.selkey);
+
+  dbg("selkey %s\n", inp->selkey);
+
   inp->M_DUP_SEL=th.M_DUP_SEL;
   inp->space_style=th.space_style;
   inp->flag=th.flag;
@@ -1356,6 +1388,9 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
   gboolean is_keypad = FALSE;
   gboolean shift_m = (kbstate & ShiftMask) > 0;
   gboolean ctrl_m = (kbstate & ControlMask) > 0;
+  int caps_eng_tog = tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock;
+  gboolean capslock_on = (kbstate&LockMask);
+  gboolean is_dayi = !strncmp(cur_inmd->filename, "dayi", 4);
 
   bzero(seltab_phrase, sizeof(seltab_phrase));
 
@@ -1364,6 +1399,13 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
   if (!cur_inmd)
     return 0;
 
+  if (caps_eng_tog) {
+	gboolean new_tsin_pho_mode =!capslock_on;
+	if (current_CS->tsin_pho_mode != new_tsin_pho_mode) {
+      current_CS->tsin_pho_mode = new_tsin_pho_mode;
+	}
+  }
+
   if ((kbstate & (Mod1Mask|Mod4Mask|Mod5Mask|ControlMask))==ControlMask
      && key>='1' && key<='9' && ggg.gbufN) {
     save_gtab_buf_phrase(key);
@@ -1373,6 +1415,10 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
   if (ggg.gbufN && key==XK_Tab)
     return 1;
 
+   if ((key==XK_Shift_L||key==XK_Shift_R) && !key_press_time) {
+     key_press_time = current_time();
+	 key_press_time_ctrl = 0;
+   } else
   if ((key==XK_Control_L||key==XK_Control_R) && !key_press_time_ctrl && tss.pre_selN) {
     key_press_time_ctrl = current_time();
     return TRUE;
@@ -1391,11 +1437,11 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
     GTK_WIDGET_VISIBLE(gwin_pho))
      hide_win_pho();
 
-  if (gtab_capslock_in_eng && (kbstate&LockMask) && !BITON(cur_inmd->flag, FLAG_KEEP_KEY_CASE)) {
+  if (!tsin_pho_mode()) {
     if (key < 0x20 || key>=0x7f)
       goto shift_proc;
 
-    if (gcin_capslock_lower)
+    if (capslock_on && gcin_capslock_lower)
       case_inverse((KeySym *)&key, shift_m);
 
     if (ggg.gbufN)
@@ -1421,7 +1467,7 @@ gboolean feedkey_gtab(KeySym key, int kbstate)
 
 
 shift_proc:
-  if (shift_m && !strchr(cur_inmd->selkey, key) && !ggg.more_pg &&
+  if (shift_m && !strchr(cur_inmd->selkey, key) && !ggg.more_pg && key>=' ' &&
        key!='*' && (key!='?' || gtab_shift_phrase_key && !ggg.ci)) {
     if (gtab_shift_phrase_key) {
       if (tss.pre_selN && shift_char_proc(key, kbstate))
@@ -1744,7 +1790,12 @@ direct_select:
     default:
 next:
 
-      if (shift_m && tss.pre_selN && shift_char_proc(key, kbstate))
+      if (key < 0x7f)
+        inkey= cur_inmd->keymap[key];
+      else
+        inkey = 0;
+
+      if (shift_m && !inkey && tss.pre_selN && shift_char_proc(key, kbstate))
         return TRUE;
 
       clear_gtab_input_error_color();
@@ -1760,7 +1811,7 @@ next:
           } else
             return 0;
         }
-        if (!strncmp(cur_inmd->filename, "dayi", 4)) {
+        if (is_dayi) {
           key = key - XK_KP_0 + '0';
           is_keypad = TRUE;
         }
@@ -1794,18 +1845,8 @@ keypad_proc:
         return 0;
       }
 
-#if 0
-      if (key > 0x7f) {
-        return 0;
-      }
-#endif
 
 
-      if (key < 0x7f)
-        inkey= cur_inmd->keymap[key];
-      else
-        inkey = 0;
-
 //        dbg("ggg.spc_pressed %d %d %d is_keypad:%d\n", ggg.spc_pressed, ggg.last_full, cur_inmd->MaxPress, is_keypad);
 
 #if 1 // for dayi, testcase :  6 space keypad6
