@@ -276,7 +276,10 @@ void toggle_half_full_char();
 static void cb_activate_state(GtkStatusIcon *status_icon, gpointer user_data)
 {
 //  dbg("cb_activate\n");
-  toggle_half_full_char();
+  if (gcin_tray_hf_win_kbm)
+    kbm_toggle();
+  else
+    toggle_half_full_char();
 }
 
 
@@ -369,11 +372,18 @@ void load_tray_icon_win32()
     iconame=inmd[current_CS->in_method].icon;
   }
 
+//  dbg("tsin_pho_mode() %d\n", tsin_pho_mode());
+
   char tt[32];
-  if (current_CS && (current_method_type()==method_type_TSIN || current_method_type()==method_type_ANTHY) &&current_CS->im_state == GCIN_STATE_CHINESE && !tsin_pho_mode()) {
-    strcpy(tt, "en-");
-    strcat(tt, iconame);
-    iconame = tt;
+  if (current_CS && current_CS->im_state == GCIN_STATE_CHINESE && !tsin_pho_mode()) {
+	 if ((current_method_type()==method_type_TSIN || current_method_type()==method_type_MODULE)) {
+		strcpy(tt, "en-");
+		strcat(tt, iconame);
+     } else {
+		strcpy(tt, "en-tsin.png");
+     }
+
+	 iconame = tt;
   }
 
 //  dbg("iconame %s\n", iconame);
