@@ -40,20 +40,19 @@ gboolean tsin_pho_mode()
   return current_CS && current_CS->tsin_pho_mode;
 }
 
-
 void set_tsin_pho_mode0(ClientState *cs)
 {
+  if (!cs)
+    return;
   cs->tsin_pho_mode = 1;
 }
 
-
-void set_tsin_pho_mode(ClientState *cs)
+void set_tsin_pho_mode()
 {
   set_tsin_pho_mode0(current_CS);
   show_tsin_stat();
 }
 
-
 gboolean tsin_cursor_end()
 {
   return tss.c_idx==tss.c_len;
@@ -300,12 +299,11 @@ static int get_in_area_pho_tsin_str(char *out)
   return outN;
 }
 
-void clear_chars_all(),clear_tsin_line();
+void clear_chars_all();
 
 static void clear_match()
 {
   tss.ph_sta=-1;
-  clear_tsin_line();
 }
 
 static void clr_ch_buf()
@@ -326,7 +324,6 @@ static void clear_ch_buf_sel_area()
   tss.full_match = FALSE;
   clr_ch_buf();
   drawcursor();
-  clear_tsin_line();
 }
 
 static void close_selection_win();
@@ -814,7 +811,7 @@ static void disp_current_sel_page()
     if (idx < phrase_count + pho_count) {
       int v = idx - phrase_count + tss.startf;
       char *tstr = pho_idx_str(v);
-      set_sele_text(phrase_count + pho_count, i, tstr, strlen(tstr));
+      set_sele_text(phrase_count + pho_count, i, tstr, -1);
     } else
       break;
   }
@@ -923,9 +920,11 @@ void show_button_pho(gboolean bshow);
 
 void tsin_set_eng_ch(int nmod)
 {
+//  dbg("tsin_set_eng_ch %d\n", nmod);
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   if (current_CS)
     current_CS->tsin_pho_mode = nmod;
 
@@ -974,30 +973,15 @@ void hide_pre_sel()
 }
 
 
-
-void draw_underline(int index);
-
-static void draw_ul(int start, int stop)
+static void call_tsin_parse()
 {
-  if (test_mode)
-    return;
-
-  int i;
-  for(i=start; i < stop; i++)
-    draw_underline(i);
+  prbuf();
+  tsin_parse();
+  prbuf();
 }
 
 void disp_ph_sta_idx(int idx)
 {
-#if 0
-//  dbg("tss.ph_sta:%d\n", tss.ph_sta);
-  clear_tsin_line();
-
-  if (tss.ph_sta < 0)
-    return;
-
-  draw_ul(idx, tss.c_idx);
-#endif
 }
 
 void disp_ph_sta()
@@ -1132,6 +1116,7 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
     ch_pho_cpy(&tss.chpho[idx], str, pho, len);
     set_chpho_ch(&tss.chpho[idx], str, len, FALSE);
     set_fixed(idx, len);
+    tss.chpho[idx].flag |= FLAG_CHPHO_PHRASE_USER_HEAD;
     tss.c_idx=idx + len;
     tss.chpho[tss.c_idx - 1].flag |= FLAG_CHPHO_PHRASE_TAIL;
 
@@ -1146,6 +1131,8 @@ gboolean add_to_tsin_buf_phsta(char *str, phokey_t *pho, int len)
     disp_ph_sta();
     hide_pre_sel();
     tss.ph_sta=-1;
+    call_tsin_parse();
+
     return 1;
 }
 
@@ -1232,7 +1219,7 @@ static gboolean pre_sel_handler(KeySym xkey)
   int c = shift_key_idx(pho_selkey, xkey);
   if (c < 0) {
     close_selection_win();
-	return FALSE;
+    return FALSE;
   }
   return tsin_sele_by_idx(c);
 }
@@ -1256,8 +1243,8 @@ static gboolean pre_punctuation_sub(KeySym xkey, char shift_punc[], unich_t *cha
 
 static gboolean pre_punctuation(KeySym xkey)
 {
-  static char shift_punc[]="<>?:\"{}!";
-  static unich_t *chars[]={_L("，"),_L("。"),_L("？"),_L("："),_L("；"),_L("『"),_L("』"),_L("！")};
+  static char shift_punc[]="<>?:\"{}!_";
+  static unich_t *chars[]={_L("，"),_L("。"),_L("？"),_L("："),_L("；"),_L("『"),_L("』"),_L("！"),_L("——")};
   return pre_punctuation_sub(xkey, shift_punc, chars);
 }
 
@@ -1272,12 +1259,6 @@ static gboolean pre_punctuation_hsu(KeySym xkey)
 int inph_typ_pho(KeySym newkey);
 gint64 current_time();
 
-static void call_tsin_parse()
-{
-  prbuf();
-  tsin_parse();
-  prbuf();
-}
 
 KeySym keypad_proc(KeySym xkey)
 {
@@ -1340,7 +1321,6 @@ void tsin_scan_pre_select(gboolean b_incr);
 static int cursor_backspace()
 {
         close_selection_win();
-        clear_tsin_line();
         poo.ityp3_pho=0;
         tss.pre_selN = 0;
         gboolean pho_cleared;
@@ -1366,6 +1346,9 @@ static int cursor_backspace()
         }
 
         if (pho_cleared) {
+		  if (typ_pho_empty())
+			  bzero(poo.inph, sizeof(poo.inph));
+
           disp_in_area_pho_tsin();
           tsin_scan_pre_select(TRUE);
 
@@ -1516,12 +1499,19 @@ int feedkey_pp(KeySym xkey, int kbstate)
   int caps_eng_tog = tsin_chinese_english_toggle_key == TSIN_CHINESE_ENGLISH_TOGGLE_KEY_CapsLock;
   int status=0;
 
+
 //  dbg("feedkey_pp %x %x\n", xkey, kbstate);
 //  if (xkey=='1')
 //    dbg("aaa\n");
 
-  if (caps_eng_tog)
-    current_CS->tsin_pho_mode = !(kbstate&LockMask);
+  if (caps_eng_tog) {
+	gboolean new_tsin_pho_mode =!(kbstate&LockMask);
+	if (current_CS->tsin_pho_mode != new_tsin_pho_mode) {
+	  close_selection_win();
+      current_CS->tsin_pho_mode = new_tsin_pho_mode;
+	  tsin_set_eng_ch(new_tsin_pho_mode);
+	}
+  }
 
 //  key_press_time = 0;
 
@@ -1552,7 +1542,8 @@ int feedkey_pp(KeySym xkey, int kbstate)
      gboolean is_ascii = (xkey>=' ' && xkey<0x7f) && !ctrl_m;
 
      if (caps_eng_tog && is_ascii) {
-       case_inverse(&xkey, shift_m);
+	   if (gcin_capslock_lower)
+         case_inverse(&xkey, shift_m);
        send_ascii(xkey);
        return 1;
      }
@@ -1638,8 +1629,14 @@ int feedkey_pp(KeySym xkey, int kbstate)
         return cursor_right();
      case XK_Caps_Lock:
         if (caps_eng_tog) {
+#if 0
           close_selection_win();
+#if UNIX
           tsin_toggle_eng_ch();
+#else
+		  tsin_set_eng_ch(!(kbstate&LockMask));
+#endif
+#endif
           return 1;
         } else
           return 0;
@@ -2203,7 +2200,9 @@ fin:
     attr[0].ofs1=tss.c_len;
     attrN++;
 
-    if (tss.c_idx < tss.c_len) {
+
+    // for firefox 4
+    if (gcin_edit_display_ap_only() && tss.c_idx < tss.c_len) {
       attr[1].ofs0=tss.c_idx;
       attr[1].ofs1=tss.c_idx+1;
       attr[1].flag=GCIN_PREEDIT_ATTR_FLAG_REVERSE;
@@ -2212,7 +2211,7 @@ fin:
   }
 
   *cursor = tss.c_idx;
-#if WIN32
+#if WIN32 || 1
   *sub_comp_len = !typ_pho_empty();
   if (gwin1 && GTK_WIDGET_VISIBLE(gwin1))
     *sub_comp_len|=2;
