@@ -3,10 +3,13 @@
 #include "win-sym.h"
 #include "gst.h"
 
+#if WIN32
 extern gboolean test_mode;
+#endif
 static int current_gcin_inner_frame;
 static int current_gtab_in_row1;
 static int current_gtab_vertical_select;
+extern int destroy_window;
 
 GtkWidget *gwin_gtab;
 static GtkWidget *top_bin;
@@ -14,10 +17,8 @@ static GtkWidget *label_full, *label_gtab_sele, *label_gtab_pre_sel;
 static GtkWidget *label_gtab = NULL;
 static GtkWidget *label_input_method_name;
 static GtkWidget *label_key_codes;
-#if WIN32
 char str_key_codes[128];
 gboolean better_key_codes;
-#endif
 static GtkWidget *box_gtab_im_name;
 static GtkWidget *hbox_row2;
 static GtkWidget *label_page;
@@ -25,7 +26,6 @@ static GtkWidget *label_edit;
 static GdkColor better_color;
 gboolean last_cursor_off;
 
-Window xwin_gtab;
 void set_label_space(GtkWidget *label);
 void minimize_win_gtab();
 gboolean win_size_exceed(GtkWidget *win), gtab_phrase_on(), gcin_edit_display_ap_only();
@@ -38,6 +38,8 @@ unich_t cht_halt_str[]=_L("半");
 
 static void adj_gtab_win_pos()
 {
+  if (!gwin_gtab)
+    return;
   if (win_size_exceed(gwin_gtab))
     move_win_gtab(current_in_win_x, current_in_win_y);
 }
@@ -48,13 +50,12 @@ void disp_gtab(char *str)
   if (test_mode)
     return;
 #endif
-   if (!label_gtab)
+  if (!label_gtab)
      return;
   if (str) {
     gtk_widget_show(label_gtab);
     gtk_label_set_text(GTK_LABEL(label_gtab), str);
-  }
-  else
+  } else
     gtk_widget_hide(label_gtab);
 
   adj_gtab_win_pos();
@@ -75,8 +76,10 @@ void set_gtab_input_error_color()
 
 void clear_gtab_input_error_color()
 {
+#if WIN32
   if (test_mode)
     return;
+#endif
   set_gtab_input_color(NULL);
 }
 
@@ -96,16 +99,12 @@ void gtab_disp_empty(char *tt, int N)
 
 void clear_gtab_in_area()
 {
-#if 0
-  disp_gtab(NULL);
-#else
   if (!cur_inmd)
     return;
   char tt[64];
   tt[0]=0;
   gtab_disp_empty(tt, win_gtab_max_key_press);
   disp_gtab(tt);
-#endif
 }
 
 
@@ -187,18 +186,19 @@ void disp_gtab_sel(char *s)
 
 void set_key_codes_label(char *s, int better)
 {
+//  dbg("set_key_codes_label %s %x\n", s, label_key_codes);
   if (!label_key_codes)
     return;
-
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   if (s && strlen(s)) {
     if (hbox_row2 && (!gtab_hide_row2 || ggg.wild_mode
-#if WIN32
-		|| str_key_codes[0]
+#if WIN32 || 1
+        || (str_key_codes[0])
 #endif
-		)) {
+      )) {
       gtk_widget_show(hbox_row2);
     }
   } else {
@@ -213,7 +213,10 @@ void set_key_codes_label(char *s, int better)
     gtk_widget_modify_fg(label_key_codes, GTK_STATE_NORMAL, NULL);
 
   gtk_label_set_text(GTK_LABEL(label_key_codes), s);
-#if WIN32
+  if (s && s[0])
+    gtk_widget_show(label_key_codes);
+
+#if WIN32 || 1
   better_key_codes = better;
   if (s && s != str_key_codes)
     strcpy(str_key_codes, s);
@@ -234,6 +237,8 @@ void show_win_sym();
 
 void move_win_gtab(int x, int y)
 {
+  if (!gwin_gtab)
+    return;
 //  dbg("move_win_gtab %d %d\n", x, y);
   get_win_size(gwin_gtab, &win_xl, &win_yl);
 
@@ -265,6 +270,8 @@ void set_gtab_input_method_name(char *s)
   gtk_label_set_text(GTK_LABEL(label_input_method_name), s);
 }
 
+gboolean use_tsin_sel_win();
+void init_tsin_selection_win();
 
 void create_win_gtab()
 {
@@ -282,10 +289,12 @@ void create_win_gtab()
 #if UNIX
   GdkWindow *gdkwin = gtk_widget_get_window(gwin_gtab);
   set_no_focus(gwin_gtab);
-  xwin_gtab = GDK_WINDOW_XWINDOW(gdkwin);
 #else
   win32_init_win(gwin_gtab);
 #endif
+
+  if (use_tsin_sel_win())
+    init_tsin_selection_win();
 }
 
 void create_win_sym();
@@ -324,7 +333,6 @@ void show_hide_label_edit()
 
   if (gcin_edit_display_ap_only() || !gtab_phrase_on()) {
     gtk_widget_hide(label_edit);
-    return;
   } else
     gtk_widget_show(label_edit);
 }
@@ -332,9 +340,10 @@ void show_hide_label_edit()
 
 void disp_label_edit(char *str)
 {
+#if WIN32
   if (test_mode)
     return;
-
+#endif
   if (!label_edit)
     return;
 
@@ -384,6 +393,7 @@ static void destroy_if_necessary()
   hbox_row2 = NULL;
 }
 
+void mod_bg_all(GtkWidget *lab, GdkColor *col);
 
 void create_win_gtab_gui_simple()
 {
@@ -394,7 +404,7 @@ void create_win_gtab_gui_simple()
   if (top_bin)
     return;
 
-  dbg("create_win_gtab_gui_simple\n");
+//  dbg("create_win_gtab_gui_simple\n");
 
   last_cursor_off = FALSE;
 
@@ -409,6 +419,7 @@ void create_win_gtab_gui_simple()
     gtk_container_add (GTK_CONTAINER(gwin_gtab), frame);
     gtk_container_set_border_width (GTK_CONTAINER (gwin_gtab), 0);
     gtk_container_add (GTK_CONTAINER (frame), vbox_top);
+    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
   } else {
     gtk_container_add (GTK_CONTAINER(gwin_gtab), vbox_top);
     top_bin = vbox_top;
@@ -451,7 +462,7 @@ void create_win_gtab_gui_simple()
       gtk_box_pack_start (GTK_BOX (hbox_row1), align, FALSE, FALSE, 0);
   }
 
-  if (gtab_phrase_pre_select) {
+  if (gtab_phrase_pre_select && !use_tsin_sel_win()) {
     label_gtab_pre_sel = gtk_label_new(NULL);
     set_label_font_size(label_gtab_pre_sel, gcin_font_size_tsin_presel);
     gtk_box_pack_start (GTK_BOX (vbox_top), label_gtab_pre_sel, FALSE, FALSE, 0);
@@ -471,6 +482,7 @@ void create_win_gtab_gui_simple()
   gtk_container_set_border_width (GTK_CONTAINER (event_box_input_method_name), 0);
 
   GtkWidget *frame_input_method_name = gtk_frame_new(NULL);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame_input_method_name), GTK_SHADOW_OUT);
   gtk_container_add (GTK_CONTAINER (event_box_input_method_name), frame_input_method_name);
   gtk_container_set_border_width (GTK_CONTAINER (frame_input_method_name), 0);
 
@@ -489,6 +501,7 @@ void create_win_gtab_gui_simple()
 
   if (!gcin_on_the_spot_key) {
     GtkWidget *frame_gtab = gtk_frame_new(NULL);
+    gtk_frame_set_shadow_type(GTK_FRAME(frame_gtab), GTK_SHADOW_OUT);
     gtk_container_set_border_width (GTK_CONTAINER (frame_gtab), 0);
     gtk_container_add (GTK_CONTAINER (event_box_gtab), frame_gtab);
     g_signal_connect(G_OBJECT(event_box_gtab),"button-press-event",
@@ -511,7 +524,10 @@ void create_win_gtab_gui_simple()
   }
 
   label_key_codes  = gtk_label_new(NULL);
+#if 0
   gtk_label_set_selectable(GTK_LABEL(label_key_codes), TRUE);
+  mod_bg_all(label_key_codes, NULL);
+#endif
   gtk_box_pack_start (GTK_BOX (hbox_row2), label_key_codes, FALSE, FALSE, 2);
 
   label_page  = gtk_label_new(NULL);
@@ -538,6 +554,8 @@ void create_win_gtab_gui_simple()
 
   if (gtab_hide_row2)
     gtk_widget_hide(hbox_row2);
+
+  minimize_win_gtab();
 }
 
 
@@ -571,7 +589,7 @@ void show_win_gtab()
 
   create_win_gtab();
   create_win_gtab_gui();
-#if WIN32
+#if WIN32 || 1
   // window was destroyed
   if (gcin_pop_up_win)
     set_key_codes_label(str_key_codes, better_key_codes);
@@ -589,18 +607,21 @@ void show_win_gtab()
     return;
 
 //  dbg("show_win_gtab()\n");
-#if UNIX
-  if (!GTK_WIDGET_VISIBLE(gwin_gtab))
-#endif
+#if WIN32
     gtk_widget_show(gwin_gtab);
+#endif
 
-#if UNIX
+#if UNIX && 0
   if (current_CS->b_raise_window)
 #endif
     gtk_window_present(GTK_WINDOW(gwin_gtab));
 
-#if WIN32
-	move_win_gtab(current_in_win_x, current_in_win_y);
+#if WIN32 || 1
+  move_win_gtab(current_in_win_x, current_in_win_y);
+#endif
+
+#if UNIX
+  gtk_widget_show(gwin_gtab);
 #endif
 
   show_win_sym();
@@ -610,13 +631,11 @@ void show_win_gtab()
 void hide_win_sym();
 void close_gtab_pho_win();
 
-void destroy_win_gtab()
+static void destroy_top_bin()
 {
-  if (!gwin_gtab)
+  if (!top_bin)
     return;
-
-  gtk_widget_destroy(gwin_gtab);
-  gwin_gtab = NULL;
+  gtk_widget_destroy(top_bin);
   top_bin = NULL;
   hbox_row2 = NULL;
   label_full=NULL;
@@ -627,6 +646,16 @@ void destroy_win_gtab()
   box_gtab_im_name = NULL;
   label_page = NULL;
   label_edit = NULL;
+  label_gtab_pre_sel = NULL;
+}
+
+void destroy_win_gtab()
+{
+  if (!gwin_gtab)
+    return;
+  destroy_top_bin();
+  gtk_widget_destroy(gwin_gtab);
+  gwin_gtab = NULL;
 }
 
 void hide_win_gtab()
@@ -642,11 +671,12 @@ void hide_win_gtab()
 
 //  dbg("hide_win_gtab\n");
   if (gwin_gtab) {
-#if UNIX
-    gtk_widget_hide(gwin_gtab);
-#else
-	  destroy_win_gtab();
-#endif
+    if (destroy_window)
+      destroy_win_gtab();
+    else {
+      gtk_widget_hide(gwin_gtab);
+      destroy_top_bin();
+    }
   }
 
   close_gtab_pho_win();
@@ -659,12 +689,10 @@ void minimize_win_gtab()
   if (test_mode)
     return;
 #endif
-//  if (!GTK_WIDGET_VISIBLE(gwin_gtab))
-//    return;
+  if (!gwin_gtab)
+    return;
 
-//  dbg("minimize_win_gtab\n");
-  // bug in GTK, in key disp is incomplete
-  gtk_window_resize(GTK_WINDOW(gwin_gtab), 20, 20);
+  gtk_window_resize(GTK_WINDOW(gwin_gtab), 10, 10);
 }
 
 
@@ -704,8 +732,10 @@ char *get_full_str()
 
 void win_gtab_disp_half_full()
 {
+#if WIN32
   if (test_mode)
     return;
+#endif
 
   if (label_full) {
     if (current_CS->im_state == GCIN_STATE_CHINESE && (!current_CS->b_half_full_char))
@@ -731,12 +761,19 @@ void disp_gtab_pre_sel(char *s)
   adj_gtab_win_pos();
 }
 
+void hide_selections_win();
+
 void hide_gtab_pre_sel()
 {
+  if (use_tsin_sel_win())
+    hide_selections_win();
+
 //  dbg("hide_gtab_pre_sel %d\n", tss.ctrl_pre_sel);
   tss.pre_selN = 0;
   tss.ctrl_pre_sel = FALSE;
-  gtk_widget_hide(label_gtab_pre_sel);
+  if (label_gtab_pre_sel)
+    gtk_widget_hide(label_gtab_pre_sel);
+
   minimize_win_gtab();
 
   move_win_gtab(current_in_win_x, current_in_win_y);
