@@ -15,7 +15,8 @@ enum {
   K_FILL=1,
   K_HOLD=2,
   K_PRESS=4,
-  K_AREA_R=8
+  K_AREA_R=8,
+  K_CAPSLOCK=16
 };
 
 
@@ -24,7 +25,6 @@ typedef struct {
   unich_t *enkey;
   char shift_key;
   char flag;
-  char *imstr;
   GtkWidget *lab, *but, *laben;
 } KEY;
 
@@ -39,7 +39,7 @@ static KEY keys[][COLN]={
 {{XK_Tab, _L("Tab")}, {'q',_L(" q ")},{'w',_L(" w ")},{'e',_L(" e ")},{'r',_L(" r ")},{'t',_L(" t ")}, {'y',_L(" y ")},{'u',_L(" u ")},{'i',_L(" i ")},{'o',_L(" o ")}, {'p',_L(" p ")},{'[',_L(" [ "),'{'},{']',_L(" ] "),'}'},{'\\',_L(" \\ "),'|',K_FILL},{XK_Delete,_L("Del"),0,8},{XK_End,_L("En"),0,8},
 {XK_Next,_L("P↓"),0,8}},
 
-{{XK_Caps_Lock, _L("Caps")},{'a',_L(" a ")},{'s',_L(" s ")},{'d',_L(" d ")},{'f', _L(" f ")},{'g',_L(" g ")},{'h',_L(" h ")},{'j',_L(" j ")},{'k',_L(" k ")},{'l',_L(" l ")},{';',_L(" ; "),':'},{'\'',_L(" ' "),'"'},{XK_Return,_L(" Enter "),0,1},{XK_Num_Lock,_L("Num"),0,8},{XK_KP_Add,_L(" + "),0,8}},
+{{XK_Caps_Lock, _L("Caps"), 0, K_CAPSLOCK},{'a',_L(" a ")},{'s',_L(" s ")},{'d',_L(" d ")},{'f', _L(" f ")},{'g',_L(" g ")},{'h',_L(" h ")},{'j',_L(" j ")},{'k',_L(" k ")},{'l',_L(" l ")},{';',_L(" ; "),':'},{'\'',_L(" ' "),'"'},{XK_Return,_L(" Enter "),0,1},{XK_Num_Lock,_L("Num"),0,8},{XK_KP_Add,_L(" + "),0,8}},
 
 {{XK_Shift_L,_L("  Shift  "),0,K_HOLD},{'z',_L(" z ")},{'x',_L(" x ")},{'c',_L(" c ")},{'v',_L(" v ")},{'b',_L(" b ")},{'n',_L(" n ")},{'m',_L(" m ")},{',',_L(" , "),'<'},{'.',_L(" . "),'>'},{'/',_L(" / "),'?'},{XK_Shift_R,_L(" Shift"),0,K_HOLD|K_FILL},{XK_KP_Multiply,_L(" * "),0,8},
 {XK_Up,_L("↑"),0,8}},
@@ -59,6 +59,7 @@ void mod_fg_all(GtkWidget *lab, GdkColor *col)
   gtk_widget_modify_fg(lab, GTK_STATE_PRELIGHT, col);
 }
 
+
 void send_fake_key_eve(KeySym key);
 #if WIN32
 void win32_FakeKey(UINT vk, bool key_press);
@@ -146,7 +147,7 @@ static void cb_button_release(GtkWidget *wid, KEY *k)
         if (!(keys[i][j].flag & K_PRESS))
           continue;
         keys[i][j].flag &= ~K_PRESS;
-		send_fake_key_eve2(keys[i][j].keysym, FALSE);
+                send_fake_key_eve2(keys[i][j].keysym, FALSE);
         mod_fg_all(keys[i][j].laben, NULL);
       }
     }
@@ -240,12 +241,16 @@ static void move_win_kbm()
   int ox, oy;
   GdkRectangle r;
   GtkOrientation ori;
-#if UNIX && !GTK_CHECK_VERSION(2,91,0)
   int szx, szy;
+#if UNIX
   if (tray_da_win) {
     gdk_window_get_origin(tray_da_win, &ox, &oy);
+#if !GTK_CHECK_VERSION(2,91,0)
     gdk_drawable_get_size(tray_da_win, &szx, &szy);
-
+#else
+    szx = gdk_window_get_width(tray_da_win);
+    szy = gdk_window_get_height(tray_da_win);
+#endif
     if (oy<height) {
       oy = szy;
     } else {
@@ -296,19 +301,11 @@ void show_win_kbm()
 static char   shift_chars[]="~!@#$%^&*()_+{}|:\"<>?";
 static char shift_chars_o[]="`1234567890-=[]\\;',./";
 
-
 #include "pho.h"
 
-static void set_kbm_key(KeySym keysym, char *str)
+static KEY *get_keys_ent(KeySym keysym)
 {
   int i;
-
-  if (!gwin_kbm)
-    return;
-
-  if (!(str[0] & 0x80) && strlen(str)==1)
-    return;
-
   for(i=0;i<keysN;i++) {
     int j;
     for(j=0;j<COLN;j++) {
@@ -322,22 +319,41 @@ static void set_kbm_key(KeySym keysym, char *str)
 
       if (keys[i][j].keysym!=keysym)
         continue;
-
-      GtkWidget *lab = keys[i][j].lab;
-      char *t = (char *)gtk_label_get_text(GTK_LABEL(lab));
-      char tt[64];
-
-      if (t && strcmp(t, str)) {
-        strcat(strcpy(tt, t), str);
-        str = tt;
-      }
-
-      if (lab) {
-        gtk_label_set_text(GTK_LABEL(lab), str);
-        set_label_font_size(lab, gcin_font_size_win_kbm);
-      }
+      return &keys[i][j];
     }
   }
+
+  return NULL;
+}
+
+static void set_kbm_key(KeySym keysym, char *str)
+{
+  int i;
+
+  if (!gwin_kbm)
+    return;
+#if 0
+  if (strlen(str)==1 && !(str[0] & 0x80))
+    return;
+#endif
+
+  KEY *p = get_keys_ent(keysym);
+  if (!p)
+    return;
+
+  GtkWidget *lab = p->lab;
+  char *t = (char *)gtk_label_get_text(GTK_LABEL(lab));
+  char tt[64];
+
+  if (t && strcmp(t, str)) {
+    strcat(strcpy(tt, t), str);
+    str = tt;
+  }
+
+  if (lab) {
+    gtk_label_set_text(GTK_LABEL(lab), str);
+    set_label_font_size(lab, gcin_font_size_win_kbm);
+  }
 }
 
 static void clear_kbm()
@@ -356,6 +372,25 @@ static void clear_kbm()
   }
 }
 
+static void disp_shift_keys()
+{
+      int i;
+      for(i=127; i > 0; i--) {
+        char tt[64];
+          KEY *p = get_keys_ent(i);
+          if (p && p->shift_key) {
+            char *t = (char *)gtk_label_get_text(GTK_LABEL(p->lab));
+            if (t && t[0])
+              continue;
+//            dbg("zzz %c %s\n",i, tt);
+            tt[0]=p->shift_key;
+            tt[1]=0;
+            set_kbm_key(i, tt);
+          }
+      }
+}
+
+
 void update_win_kbm()
 {
   if (!current_CS || !gwin_kbm)
@@ -408,31 +443,45 @@ void update_win_kbm()
         }
 
         if (!ttN)
-          continue;
-
+         continue;
         set_kbm_key(i, tt);
       }
+
+      disp_shift_keys();
+
       break;
     case method_type_INT_CODE:
-#if USE_ANTHY
-    case method_type_ANTHY:
-#endif
+    case method_type_MODULE:
       break;
     default:
       if (!cur_inmd || !cur_inmd->DefChars)
         return;
 
+      int loop;
+      for(loop=0;loop<2;loop++)
       for(i=127; i > 0; i--) {
+        char tt[64];
         char k=cur_inmd->keymap[i];
         if (!k)
           continue;
 
         char *keyname = &cur_inmd->keyname[k * CH_SZ];
-        if (!keyname)
+        if (!keyname[0])
           continue;
 
-        char tt[64];
+        if (loop==0 && !(keyname[0]&0x80))
+          continue;
 
+        if (loop==1) {
+          KEY *p = get_keys_ent(i);
+          char *t = (char *)gtk_label_get_text(GTK_LABEL(p->lab));
+          if (t && t[0]) {
+            continue;
+          }
+        }
+
+
+        tt[0]=0;
         if (keyname[0] & 128)
           utf8cpy(tt, keyname);
         else {
@@ -441,9 +490,12 @@ void update_win_kbm()
           tt[2]=0;
         }
 
+//        dbg("%c '%s'\n", i, tt);
         set_kbm_key(i, tt);
       }
 
+      disp_shift_keys();
+
       break;
   }
 
@@ -464,3 +516,17 @@ void hide_win_kbm()
   win_kbm_on = 0;
   gtk_widget_hide(gwin_kbm);
 }
+
+extern gboolean old_capslock_on;
+
+void win_kbm_disp_caplock()
+{
+  KEY *p = get_keys_ent(XK_Caps_Lock);
+
+  if (old_capslock_on) {
+ //   dbg("lock...\n");
+    mod_fg_all(p->laben, &red);
+  } else {
+    mod_fg_all(p->laben, NULL);
+  }
+}
