@@ -6,7 +6,6 @@ static int current_pho_in_row1;
 
 GtkWidget *gwin_pho;
 static GtkWidget *top_bin, *hbox_row2;
-Window xwin_pho;
 static GtkWidget *label_pho_sele;
 static GtkWidget *label_pho;
 static GtkWidget *label_full;
@@ -26,6 +25,7 @@ void move_win_pho(int x, int y);
 void get_win_size(GtkWidget *win, int *width, int *height)
 {
   GtkRequisition sz;
+  sz.width = sz.height = 0;
   gtk_widget_size_request(GTK_WIDGET(win), &sz);
   *width = sz.width;
   *height = sz.height;
@@ -43,7 +43,7 @@ gboolean win_size_exceed(GtkWidget *win)
 
 void disp_pho_sel(char *s)
 {
-  gtk_label_set_text(GTK_LABEL(label_pho_sele), s);
+  gtk_label_set_markup(GTK_LABEL(label_pho_sele), s);
 
   if (win_size_exceed(gwin_pho)) {
     move_win_pho(current_in_win_x, current_in_win_y);
@@ -96,6 +96,7 @@ void create_win_pho()
     return;
 
   gwin_pho = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_has_resize_grip(GTK_WINDOW(gwin_pho), FALSE);
 #if WIN32
   set_no_focus(gwin_pho);
 #endif
@@ -103,7 +104,6 @@ void create_win_pho()
   gtk_widget_realize (gwin_pho);
 #if UNIX
   GdkWindow *gdkwin = gtk_widget_get_window(gwin_pho);
-  xwin_pho = GDK_WINDOW_XWINDOW(gdkwin);
   set_no_focus(gwin_pho);
 #else
   win32_init_win(gwin_pho);
@@ -147,6 +147,7 @@ void create_win_pho_gui_simple()
     gtk_container_set_border_width (GTK_CONTAINER (frame), 0);
     gtk_container_add (GTK_CONTAINER(gwin_pho), frame);
     gtk_container_add (GTK_CONTAINER (frame), vbox_top);
+    gtk_frame_set_shadow_type (GTK_FRAME (frame), GTK_SHADOW_ETCHED_OUT);
   } else {
     gtk_container_add (GTK_CONTAINER(gwin_pho), vbox_top);
     top_bin = vbox_top;
@@ -172,11 +173,7 @@ void create_win_pho_gui_simple()
   hbox_row2 = gtk_hbox_new (FALSE, 0);
   /* This packs the button into the gwin_pho (a gtk container). */
   gtk_container_add (GTK_CONTAINER (vbox_top), hbox_row2);
-#if UNIX
-  label_full = gtk_label_new("全");
-#else
-  label_full = gtk_label_new("Fu");
-#endif
+  label_full = gtk_label_new(_(_L("全")));
   gtk_container_add (GTK_CONTAINER (hbox_row2), label_full);
 
 
@@ -187,6 +184,7 @@ void create_win_pho_gui_simple()
                    G_CALLBACK(mouse_button_callback), NULL);
 
   GtkWidget *frame_pho = gtk_frame_new(NULL);
+  gtk_frame_set_shadow_type(GTK_FRAME(frame_pho), GTK_SHADOW_OUT);
   gtk_container_add (GTK_CONTAINER (event_box_pho), frame_pho);
   gtk_container_set_border_width (GTK_CONTAINER (frame_pho), 0);
 
@@ -199,7 +197,7 @@ void create_win_pho_gui_simple()
     gtk_widget_set_tooltip_text (event_box_pho, _(_L("左鍵符號，右鍵設定")));
 #else
     GtkTooltips *button_gtab_tips = gtk_tooltips_new ();
-    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_gtab_tips), event_box_pho, _(_L"左鍵符號，右鍵設定"),NULL);
+    gtk_tooltips_set_tip (GTK_TOOLTIPS (button_gtab_tips), event_box_pho, _(_L("左鍵符號，右鍵設定")),NULL);
 #endif
   }
 
@@ -239,6 +237,14 @@ void show_win_pho()
   if (gcin_pop_up_win && !pho_has_input())
     return;
 
+#if UNIX
+  if (!GTK_WIDGET_VISIBLE(gwin_pho))
+#endif
+  {
+    gtk_widget_show(gwin_pho);
+    move_win_pho(win_x, win_y);
+  }
+
   gtk_widget_show(gwin_pho);
   if (current_CS->b_raise_window)
     gtk_window_present(GTK_WINDOW(gwin_pho));
@@ -280,10 +286,10 @@ void win_pho_disp_half_full()
 {
   gtk_label_set_text(GTK_LABEL(label_pho), get_full_str());
 
-  if (current_CS->im_state == GCIN_STATE_CHINESE && current_CS->b_half_full_char) {
-    gtk_widget_show(label_full);
-  } else
+  if (current_CS->im_state == GCIN_STATE_CHINESE && (!current_CS->b_half_full_char))
     gtk_widget_hide(label_full);
+  else
+    gtk_widget_show(label_full);
 
   minimize_win_pho();
 }
