@@ -185,12 +185,10 @@ void add_to_tsin_buf_str(char *str);
 
 extern int c_len;
 extern short gbufN;
-static void cb_button_sym(GtkButton *button, char *str)
+static void cb_button_sym(GtkButton *button, GtkWidget *label)
 {
-  phokey_t pho[256];
-  bzero(pho, sizeof(pho));
-
 //  dbg("cb_button_sym\n");
+  char *str = (char *) gtk_label_get_text(GTK_LABEL(label));
 
 #if USE_TSIN
   if (current_method_type() == method_type_TSIN && current_CS->im_state == GCIN_STATE_CHINESE) {
@@ -198,9 +196,9 @@ static void cb_button_sym(GtkButton *button, char *str)
     if (tsin_cursor_end()) {
       flush_tsin_buffer();
       output_buffer_call_back();
-	} else {
+    } else {
       force_preedit_shift();
-	}
+    }
   }
   else
 #endif
@@ -224,10 +222,8 @@ static void cb_button_sym(GtkButton *button, char *str)
        tsin_reset_in_pho();
        break;
 #endif
-#if USE_ANTHY
-    case method_type_ANTHY:
+    case method_type_MODULE:
        break;
-#endif
     default:
        reset_gtab_all();
        break;
@@ -341,6 +337,36 @@ static void destory_win()
   gwin_sym = NULL;
 }
 
+static void disp_win_sym()
+{
+  syms = pages[idx].syms;
+  symsN = pages[idx].symsN;
+  destory_win();
+//  win_sym_enabled = 0;
+  create_win_sym();
+}
+
+gboolean win_sym_page_up()
+{
+  if (!win_sym_enabled)
+	  return FALSE;
+  idx--;
+  if (idx < 0)
+  idx = pagesN - 1;
+  disp_win_sym();
+  return TRUE;
+}
+
+gboolean win_sym_page_down()
+{
+//  dbg("win_sym_page_down\n");
+  if (!win_sym_enabled)
+    return FALSE;
+  idx = (idx+1) % pagesN;
+  disp_win_sym();
+  return TRUE;
+}
+
 
 static gboolean button_scroll_event(GtkWidget *widget,GdkEventScroll *event, gpointer user_data)
 {
@@ -349,25 +375,20 @@ static gboolean button_scroll_event(GtkWidget *widget,GdkEventScroll *event, gpo
 
   switch (event->direction) {
     case GDK_SCROLL_UP:
-      idx--;
-      if (idx < 0)
-        idx = pagesN - 1;
+	  win_sym_page_up();
       break;
     case GDK_SCROLL_DOWN:
-      idx = (idx+1) % pagesN;
+	  win_sym_page_down();
       break;
     default:
       break;
   }
 
-  syms = pages[idx].syms;
-  symsN = pages[idx].symsN;
-  destory_win();
-//  win_sym_enabled = 0;
-  create_win_sym();
   return TRUE;
 }
 
+
+
 static void mouse_button_callback_up_down( GtkWidget *widget,GdkEventButton *event, gpointer data)
 {
 	GdkEventScroll sc;
@@ -386,7 +407,7 @@ void create_win_sym()
     p_err("bad current_CS %d\n", current_CS->in_method);
   }
 
-  if (current_method_type() != method_type_PHO && current_method_type() != method_type_TSIN && current_method_type() != method_type_ANTHY && !cur_inmd)
+  if (current_method_type() != method_type_PHO && current_method_type() != method_type_TSIN && current_method_type() != method_type_MODULE && !cur_inmd)
     return;
 
   if (read_syms() || cur_in_method != current_CS->in_method) {
@@ -407,6 +428,7 @@ void create_win_sym()
   }
 
   gwin_sym = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_has_resize_grip(GTK_WINDOW(gwin_sym), FALSE);
 #if WIN32
   set_no_focus(gwin_sym);
 #endif
@@ -461,7 +483,7 @@ void create_win_sym()
         }
       }
 
-      g_signal_connect (G_OBJECT (button), "clicked",  G_CALLBACK (cb_button_sym), str);
+      g_signal_connect (G_OBJECT (button), "clicked",  G_CALLBACK (cb_button_sym), label);
     }
   }
 
