@@ -1,8 +1,12 @@
 #include "gcin.h"
 #include "pho.h"
+#include "gst.h"
+#include "win1.h"
 
 static GtkWidget *gwin1, *frame;
-Window xwin1;
+static char *wselkey;
+static int wselkeyN;
+//Window xwin1;
 
 #define SELEN (12)
 
@@ -11,16 +15,36 @@ static GtkWidget *eve_sele[SELEN], *eve_seleR[SELEN];
 static GtkWidget *arrow_up, *arrow_down;
 
 void hide_selections_win();
-gboolean tsin_page_up(), tsin_page_down();
+static cb_page_ud_t cb_page_up, cb_page_down;
+static int c_config;
+
+static int current_config()
+{
+  return (tsin_tail_select_key<<9) | (pho_candicate_col_N<<5) | wselkeyN << 1|
+    pho_candicate_R2L;
+}
+
+static int idx_to_x(int tN, int i)
+{
+    if (tN > pho_candicate_col_N)
+      tN = pho_candicate_col_N;
+
+    int x = i % tN;
+    if (pho_candicate_R2L)
+      x = tN - 1 - x;
+    return x;
+}
 
 static gboolean button_scroll_event_tsin(GtkWidget *widget,GdkEventScroll *event, gpointer user_data)
 {
   switch (event->direction) {
     case GDK_SCROLL_UP:
-      tsin_page_up();
+      if (cb_page_up)
+        cb_page_up();
       break;
     case GDK_SCROLL_DOWN:
-      tsin_page_down();
+      if (cb_page_down)
+        cb_page_down();
       break;
     default:
       break;
@@ -37,6 +61,7 @@ void create_win1()
     return;
 
   gwin1 = gtk_window_new (GTK_WINDOW_TOPLEVEL);
+  gtk_window_set_has_resize_grip(GTK_WINDOW(gwin1), FALSE);
 #if WIN32
   set_no_focus(gwin1);
 #endif
@@ -44,7 +69,7 @@ void create_win1()
 
 #if UNIX
   GdkWindow *gdkwin1 = gtk_widget_get_window(gwin1);
-  xwin1 = GDK_WINDOW_XWINDOW(gdkwin1);
+//  xwin1 = GDK_WINDOW_XWINDOW(gdkwin1);
   set_no_focus(gwin1);
 #else
   win32_init_win(gwin1);
@@ -54,7 +79,9 @@ void create_win1()
 }
 
 void change_win1_font(), force_preedit_shift();
-int tsin_sele_by_idx(int c);
+
+
+static cb_selec_by_idx_t cb_sele_by_idx;
 
 static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpointer data)
 {
@@ -62,12 +89,32 @@ static void mouse_button_callback( GtkWidget *widget,GdkEventButton *event, gpoi
   switch (event->button) {
     case 1:
       v = GPOINTER_TO_INT(data);
-      tsin_sele_by_idx(v);
+      if (cb_sele_by_idx)
+        cb_sele_by_idx(v);
       force_preedit_shift();
       break;
   }
 }
 
+
+static void cb_arrow_up (GtkWidget *button, gpointer user_data)
+{
+  if (cb_page_up)
+    cb_page_up();
+}
+static void cb_arrow_down (GtkWidget *button, gpointer user_data)
+{
+  if (cb_page_down)
+   cb_page_down();
+}
+
+void set_win1_cb(cb_selec_by_idx_t sele_by_idx, cb_page_ud_t page_up, cb_page_ud_t page_down)
+{
+  cb_sele_by_idx = sele_by_idx;
+  cb_page_up = page_up;
+  cb_page_down = page_down;
+}
+
 void create_win1_gui()
 {
   if (frame)
@@ -79,16 +126,35 @@ void create_win1_gui()
   GtkWidget *vbox_top = gtk_vbox_new (FALSE, 0);
   gtk_container_add (GTK_CONTAINER(frame), vbox_top);
 
+  GtkWidget *eve_box_up = gtk_event_box_new();
+  gtk_box_pack_start (GTK_BOX (vbox_top), eve_box_up, FALSE, FALSE, 0);
   arrow_up = gtk_arrow_new (GTK_ARROW_UP, GTK_SHADOW_IN);
-  gtk_box_pack_start (GTK_BOX (vbox_top), arrow_up, FALSE, FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(eve_box_up), arrow_up);
+  g_signal_connect (G_OBJECT (eve_box_up), "button-press-event",
+                      G_CALLBACK (cb_arrow_up), NULL);
 
-  GtkWidget *table = gtk_table_new(SELEN, 2, FALSE);
+  int c_rowN = (wselkeyN + pho_candicate_col_N - 1) / pho_candicate_col_N * pho_candicate_col_N;
+  int tablecolN = pho_candicate_col_N;
+
+  if (!tsin_tail_select_key)
+    tablecolN *= 2;
+
+  c_config = current_config();
+
+  GtkWidget *table = gtk_table_new(c_rowN, tablecolN, FALSE);
   gtk_box_pack_start (GTK_BOX (vbox_top), table, FALSE, FALSE, 0);
 
   int i;
-  for(i=0; i < SELEN; i++) {
+  for(i=0; i < wselkeyN; i++)
+  {
+    int y = i/pho_candicate_col_N;
+    int x = idx_to_x(SELEN+1, i);
+
+    if (!tsin_tail_select_key)
+      x*=2;
+
     GtkWidget *align = gtk_alignment_new(0,0,0,0);
-    gtk_table_attach_defaults(GTK_TABLE(table),align, 0,1,i,i+1);
+    gtk_table_attach_defaults(GTK_TABLE(table),align, x,x+1,y,y+1);
     GtkWidget *event_box_pho = gtk_event_box_new();
     GtkWidget *label = gtk_label_new(NULL);
     gtk_container_add (GTK_CONTAINER (event_box_pho), label);
@@ -100,31 +166,42 @@ void create_win1_gui()
     g_signal_connect(G_OBJECT(event_box_pho),"button-press-event",
                    G_CALLBACK(mouse_button_callback), GINT_TO_POINTER(i));
 
-
-    GtkWidget *alignR = gtk_alignment_new(0,0,0,0);
-    gtk_table_attach_defaults(GTK_TABLE(table), alignR, 1,2,i,i+1);
-    GtkWidget *event_box_phoR = gtk_event_box_new();
-    GtkWidget *labelR = gtk_label_new(NULL);
-    gtk_container_add (GTK_CONTAINER (event_box_phoR), labelR);
-    labels_seleR[i] = labelR;
-    eve_seleR[i] = event_box_phoR;
-    gtk_container_add (GTK_CONTAINER (alignR), event_box_phoR);
-    gtk_label_set_justify(GTK_LABEL(labels_sele[i]),GTK_JUSTIFY_LEFT);
-    set_label_font_size(labels_seleR[i], gcin_font_size_tsin_presel);
-    g_signal_connect(G_OBJECT(event_box_phoR),"button-press-event",
-                   G_CALLBACK(mouse_button_callback), GINT_TO_POINTER(i));
+    if (!tsin_tail_select_key) {
+      GtkWidget *alignR = gtk_alignment_new(0,0,0,0);
+      gtk_table_attach_defaults(GTK_TABLE(table), alignR, x+1,x+2,y,y+1);
+      GtkWidget *event_box_phoR = gtk_event_box_new();
+      GtkWidget *labelR = gtk_label_new(NULL);
+      gtk_container_add (GTK_CONTAINER (event_box_phoR), labelR);
+      labels_seleR[i] = labelR;
+      eve_seleR[i] = event_box_phoR;
+      gtk_container_add (GTK_CONTAINER (alignR), event_box_phoR);
+      gtk_label_set_justify(GTK_LABEL(labels_sele[i]),GTK_JUSTIFY_LEFT);
+      set_label_font_size(labels_seleR[i], gcin_font_size_tsin_presel);
+      g_signal_connect(G_OBJECT(event_box_phoR),"button-press-event",
+                     G_CALLBACK(mouse_button_callback), GINT_TO_POINTER(i));
+    }
   }
 
+  GtkWidget *eve_box_down = gtk_event_box_new();
+  gtk_box_pack_start (GTK_BOX (vbox_top), eve_box_down, FALSE, FALSE, 0);
   arrow_down = gtk_arrow_new (GTK_ARROW_DOWN, GTK_SHADOW_IN);
-  gtk_box_pack_start (GTK_BOX (vbox_top), arrow_down, FALSE, FALSE, 0);
+  gtk_container_add(GTK_CONTAINER(eve_box_down), arrow_down);
+  g_signal_connect (G_OBJECT (eve_box_down), "button-press-event",
+                      G_CALLBACK (cb_arrow_down), NULL);
 
   gtk_widget_show_all(gwin1);
-  gdk_flush();
+//  gdk_flush();
   gtk_widget_hide(gwin1);
 
   change_win1_font();
 }
 
+void init_tsin_selection_win()
+{
+  create_win1();
+  create_win1_gui();
+}
+
 void clear_sele()
 {
   int i;
@@ -132,9 +209,10 @@ void clear_sele()
   if (!gwin1)
     create_win1();
 
-  for(i=0; i < SELEN; i++) {
+  for(i=0; i < wselkeyN; i++) {
     gtk_widget_hide(labels_sele[i]);
-    gtk_widget_hide(labels_seleR[i]);
+    if (labels_seleR[i])
+      gtk_widget_hide(labels_seleR[i]);
   }
 
   gtk_widget_hide(arrow_up);
@@ -148,8 +226,11 @@ void clear_sele()
 
 char *htmlspecialchars(char *s, char out[]);
 
-void set_sele_text(int i, char *text, int len)
+void set_sele_text(int tN, int i, char *text, int len)
 {
+  if (len < 0)
+    len=strlen(text);
+
   char tt[128];
   char utf8[128];
 
@@ -158,21 +239,32 @@ void set_sele_text(int i, char *text, int len)
   char uu[32], selma[128];
 
   char cc[2];
-  cc[0]=phkbm.selkey[i];
+  cc[0]=wselkey[i];
   cc[1]=0;
+  char ul[128];
+  ul[0]=0;
 
-  sprintf(selma, "<span foreground=\"%s\">%s</span>", gcin_sel_key_color, htmlspecialchars(cc, uu));
+  if (tss.sel_pho && i==tss.pho_menu_idx)
+    strcpy(ul, " foreground=\"yellow\" background=\"black\"");
+  else
+    sprintf(ul, "foreground=\"%s\"",gcin_sel_key_color);
+
+  sprintf(selma, "<span %s>%s</span>", ul, htmlspecialchars(cc, uu));
+
+  int x = idx_to_x(tN, i);
+  char *sep= x?" ":"";
 
   if (tsin_tail_select_key) {
     char vv[128];
-    snprintf(tt, sizeof(tt), "%s %s", htmlspecialchars(utf8, vv), selma);
+    snprintf(tt, sizeof(tt), "%s%s%s", sep, htmlspecialchars(utf8, vv), selma);
   } else {
     gtk_label_set_text(GTK_LABEL(labels_seleR[i]), utf8);
     gtk_widget_show(labels_seleR[i]);
-    snprintf(tt, sizeof(tt), "%s ", selma);
+    snprintf(tt, sizeof(tt), "%s%s",sep, selma);
   }
 
   gtk_widget_show(labels_sele[i]);
+//  dbg("tt %s\n", tt);
   gtk_label_set_markup(GTK_LABEL(labels_sele[i]), tt);
 }
 
@@ -180,37 +272,13 @@ void set_sele_text(int i, char *text, int len)
 static int timeout_handle;
 gboolean timeout_minimize_win1(gpointer data)
 {
-  gtk_window_resize(GTK_WINDOW(gwin1), 10, 20);
+  gtk_window_resize(GTK_WINDOW(gwin1), 10, 10);
   gtk_window_present(GTK_WINDOW(gwin1));
   timeout_handle = 0;
   return FALSE;
 }
 #endif
 
-
-void disp_selections(int x, int y)
-{
-  if (!gwin1)
-    p_err("disp_selections !gwin1");
-
-  if (!GTK_WIDGET_VISIBLE(gwin1)) {
-    gtk_widget_show(gwin1);
-  }
-
-  int win1_xl, win1_yl;
-  get_win_size(gwin1, &win1_xl, &win1_yl);
-
-  if (x + win1_xl > dpy_xl)
-    x = dpy_xl - win1_xl;
-  if (y + win1_yl > dpy_yl)
-    y = win_y - win1_yl;
-
-  gtk_window_move(GTK_WINDOW(gwin1), x, y);
-#if WIN32
-  timeout_handle = g_timeout_add(50, timeout_minimize_win1, NULL);
-#endif
-}
-
 void raise_tsin_selection_win()
 {
   if (gwin1 && GTK_WIDGET_VISIBLE(gwin1))
@@ -218,6 +286,56 @@ void raise_tsin_selection_win()
 }
 
 
+void getRootXY(Window win, int wx, int wy, int *tx, int *ty);
+void disp_selections(int x, int y)
+{
+  if (!gwin1)
+    p_err("disp_selections !gwin1");
+#if WIN32
+  if (!GTK_WIDGET_VISIBLE(gwin1)) {
+    gtk_widget_show(gwin1);
+  }
+#endif
+
+  if (y < 0) {
+	 int tx;
+	 if (gcin_edit_display_ap_only())
+		getRootXY(current_CS->client_win, current_CS->spot_location.x, current_CS->spot_location.y, &tx, &y);
+	 else
+		 y = win_y + win_yl;
+  }
+
+
+  int win1_xl, win1_yl;
+  get_win_size(gwin1, &win1_xl, &win1_yl);
+
+  if (x < 0) {
+    x = win_x + win_xl - win1_xl;
+	if (x < win_x)
+		x = win_x;
+  }
+
+  if (x + win1_xl > dpy_xl)
+    x = dpy_xl - win1_xl;
+
+  if (y + win1_yl > dpy_yl)
+    y = win_y - win1_yl;
+
+  gtk_window_move(GTK_WINDOW(gwin1), x, y);
+#if WIN32
+  timeout_handle = g_timeout_add(50, timeout_minimize_win1, NULL);
+#endif
+
+#if UNIX
+  if (!GTK_WIDGET_VISIBLE(gwin1)) {
+    gtk_widget_show(gwin1);
+  }
+#endif
+#if WIN32
+  raise_tsin_selection_win();
+#endif
+}
+
 void hide_selections_win()
 {
   if (!gwin1)
@@ -245,7 +363,6 @@ void disp_arrow_down()
   gtk_widget_show(arrow_down);
 }
 
-#if USE_TSIN
 void destroy_win1()
 {
   if (!gwin1)
@@ -254,7 +371,6 @@ void destroy_win1()
   frame=NULL;
   gwin1 = NULL;
 }
-#endif
 
 void change_win1_font()
 {
@@ -263,7 +379,7 @@ void change_win1_font()
   GdkColor fg;
   gdk_color_parse(gcin_win_color_fg, &fg);
 
-  for(i=0; i < SELEN; i++) {
+  for(i=0; i < wselkeyN; i++) {
     set_label_font_size(labels_sele[i], gcin_font_size_tsin_presel);
     set_label_font_size(labels_seleR[i], gcin_font_size_tsin_presel);
     if (labels_sele[i])
@@ -271,8 +387,28 @@ void change_win1_font()
     if (labels_seleR[i])
       gtk_widget_modify_fg(labels_seleR[i], GTK_STATE_NORMAL, gcin_win_color_use?&fg:NULL);
     change_win_bg(eve_sele[i]);
-    change_win_bg(eve_seleR[i]);
+    if (eve_seleR[i])
+      change_win_bg(eve_seleR[i]);
   }
 
   change_win_bg(gwin1);
 }
+
+void recreate_win1_if_nessary()
+{
+  if (!frame)
+    return;
+
+  dbg("%x %x\n", current_config(), c_config);
+  if (current_config() != c_config) {
+    gtk_widget_destroy(frame); frame = NULL;
+    create_win1_gui();
+  }
+}
+
+
+void set_wselkey(char *s)
+{
+  wselkey = s;
+  wselkeyN = strlen(s);
+}
